# -*- coding:utf-8 -*-
# author: John Yan  last update on 2020-06-29
# function: generate the airflow etl_script and dag file

from airflow.contrib.dagsetup.config_dagfile import *
from airflow.settings import AIRFLOW_HOME, ETL_FILE_PATH
import re
import os
import logging
import json

DRIVES = {
    'sql_file': {'color': '#b46ade', 't': 'sql'},
    'sql_str': {'color': '#b0f07c'},
    'sp': {'color': '#cdaaed'},
    'ktr': {'color': '#f9c915', 't': 'ktr'},
    'kjb': {'color': '#e9ffdb', 't': 'kjb'},
    'ktr_r': {'color': '#f4a460', 't': 'ktr'},
    'kjb_r': {'color': '#f4a460', 't': 'kjb'},
    'py': {'color': '#ffefeb', 't': 'py'},
    'link': {'color': '#70586d'},
    'validate': {'color': '#f0e4ec'},
    'dataset': {'color': '#f9c915'},
    'refresh_tableau': {'color': '#f4a460'},
    'impala_sql': {'color': '#835bf0', 't': 'sql'},
    'hive_sql': {'color': '#9486ba', 't': 'sql'},
    'gp_sql': {'color': '#cecadb', 't': 'sql'},
    'mysql_sql': {'color': '#74946e', 't': 'sql'},
    'oracle_sql': {'color': '#74946e', 't': 'sql'},
    'mssql_sql': {'color': '#74946e', 't': 'sql'},
    'starrocks_sql': {'color': '#74946e', 't': 'sql'},
    'sqoop': {'color': '#665908', 't': 'sql'},
    'sap': {'color': '#f4a460', 't': 'json'},
    'trigger': {'color': '#9cbaba'},
    'branch': {'color': '#3d7373'},
    'diy': {'color': '#c46c6c'},
    'hdfs': {'color': '#4675bf', 't': 'ktr'},
    'datax': {'color': '#4675cf', 't': 'sql'},
    'dataxx': {'color': '#4675df', 't': 'json'},
    'refresh_smc': {'color': '#6675df'},
    'refresh_quality': {'color': '#6875df'},
    'hdfsstarrocks': {'color': '#665998', 't': 'sql'},
    'kafkastarrocks': {'color': '#e8a6a6', 't': 'sql'},
    'apistarrocks': {'color': '#eea6a6', 't': 'sql'},
    'routinestarrocks': {'color': '#b4a7d6'},
}


def gen_airflow_dag(*args, **kwargs):
    """
    generate the airflow etl_script and dag file
    :param owner: 负责人
    :param project_name: 项目名,将用于DAG的唯一标识
    :param schedule_interval: cron 模式 * * * * * *
    :param schedule_rate: 每天/每周/每月
    :param job_list: 可以是数组[[]] 或 string
    :param dag_remark: 备注信息
    :param dag_sequence: airflow 格式的执行顺序,可以为空
    :return: None
    #sp|sp_name|oracle|para2_name,para3_name||
    """
    msg = ''
    owner = kwargs['owner']
    project_name = kwargs['project_name']
    description = kwargs.get('description', '')
    schedule_interval = kwargs['schedule_interval']
    schedule_rate = kwargs['schedule_rate']
    job_list = kwargs['job_list']
    dag_remark = kwargs.get('dag_remark', '')
    dag_sequence = kwargs.get('dag_sequence', '')
    retries = kwargs.get('retries', 0)
    mail = kwargs.get('mail', '')
    common_path = kwargs.get('common_path', '')
    tags = kwargs.get('tags')
    tags = tags.split(',') if tags else []
    extra_args = ''
    if mail:
        extra_args = f"'email': '{mail}'"
    # -----------------transfer job str to job list---------------------
    user_define = ''
    if isinstance(job_list, str):
        items = job_list.split('#')
        # 第一行预留给写自定义参数或代码
        user_define = items[0].strip()
        job_list = []
        # 过滤每一个
        for item in items[1:]:
            item = item.strip().split('--')[0]
            if item:
                item_lst = item.split('|')
                if len(item_lst) < 2:
                    item_lst = re.split('\s+', item)
                item_lst = list(map(lambda x: x.strip(), item_lst))
                color = DRIVES.get(item_lst[0].lower())
                if not color:
                    # 将非合法的driver当成备注
                    user_define = user_define + '\n#--' + item
                    continue
                item_lst.extend([''] * (7 - len(item_lst)))
                item_lst[-2] = item_lst[0]
                item_lst[-1] = color.get('color')
                job_list.append(item_lst)
    # 扩展job_list
    for i in range(len(job_list)):
        if job_list[i][0].endswith('_sql'):
            job_list[i][4] = job_list[i][0].replace('_sql', '')
            job_list[i][0] = 'sql_file'
    print(job_list)

    # -------------------gen job file-----------------------

    if os.path.exists(os.path.join(AIRFLOW_HOME, 'config.json')):
        with open(os.path.join(AIRFLOW_HOME, 'config.json'), 'r') as f:
            res = f.read().strip()
            if res:
                res = json.loads(res)
                common_path = res.get(owner)
    dag_str = ''
    etl_str = import_format.format(common_path=common_path)
    # extract the parameter list and construct the get variable of airflow
    # 如果用户有定义, 不采用默认取参方式
    if user_define:
        etl_str = etl_str + user_define
    # 自动不好, 取消
    # else:
    #     param_str = ''
    #     for item in job_list:
    #         param_str = param_str + ','+ item[2]
    #     param_list = set(param_str.split(','))
    #     for item in param_list:
    #         if item:
    #             etl_str = etl_str + param_format.format(param=item)

    # loop to construct jobs
    for item in job_list:
        param_str = ''
        item_param_list = item[2].split(',')
        for item_param in item_param_list:
            if item_param:
                param_str = param_str + '+' + param_str_format.format(item_param=item_param)
        if param_str:
            param_str = param_str[1:]
        else:
            param_str = "''"

        driver = item[0].lower()
        # 校验
        try:
            file_list = os.listdir(os.path.join(ETL_FILE_PATH, project_name))
        except Exception as e:
            file_list = []
        tag = DRIVES[driver].get('t')
        if tag:
            filename = item[1] + '.' + tag
            if filename not in file_list:
                msg = f'{msg}{filename},'

        # 文件类型0,文件命名1,参数2,备注3,执行数据库4
        # driver 0, filename 1, param 2, remark 3,db 4
        if driver == 'sql_file':
            etl_str = etl_str + execute_sql_format.format(project_name=project_name, filename=item[1], db=item[4],
                                                          param=item[2])
        elif driver == 'sqoop':
            etl_str = etl_str + execute_sqoop_format.format(project_name=project_name, filename=item[1], param=item[2])
        elif driver == 'hdfsstarrocks':
            etl_str = etl_str + execute_hdfsstarrocks_format.format(project_name=project_name, filename=item[1], param=item[2])
        elif driver == 'datax':
            etl_str = etl_str + execute_datax_format.format(project_name=project_name, filename=item[1], param=item[2])
        elif driver == 'kafkastarrocks':
            etl_str = etl_str + execute_kafkastarrocks_format.format(project_name=project_name, filename=item[1], param=item[2])
        elif driver == 'apistarrocks':
            etl_str = etl_str + execute_apistarrocks_format.format(project_name=project_name, filename=item[1], param=item[2])
        elif driver == 'routinestarrocks':
            etl_str = etl_str + execute_routinestarrocks_format.format(filename=item[1], label=item[2], flag=item[3])
        elif driver == 'dataxx':
            etl_str = etl_str + execute_dataxx_format.format(project_name=project_name, filename=item[1], param=item[2])
        elif driver == 'sap':
            etl_str = etl_str + execute_sap_format.format(project_name=project_name, filename=item[1], param=item[2])
        elif driver == 'validate':
            etl_str = etl_str + execute_validate_format.format(project_name=project_name, filename=item[1], db=item[4],
                                                               param=item[2])
        elif driver == 'sql_str':
            etl_str = etl_str + execute_sqlstr_format.format(filename=item[1], remark=item[3], db=item[4],
                                                             param=item[2])
        elif driver == 'sp':
            etl_str = etl_str + execute_sp_format.format(filename=item[1], param=item[2])
        elif driver == 'ktr':
            etl_str = etl_str + execute_ktr_format.format(project_name=project_name, filename=item[1],
                                                          param_str=param_str)
        elif driver == 'hdfs':
            etl_str = etl_str + execute_hdfs_format.format(project_name=project_name, filename=item[1],
                                                           param_str=param_str)
        elif driver == 'kjb':
            etl_str = etl_str + execute_kjb_format.format(project_name=project_name, filename=item[1],
                                                          param_str=param_str)
        elif driver == 'ktr_r':
            etl_str = etl_str + execute_ktr_r_format.format(project_name=project_name, filename=item[1],
                                                            param_str=param_str)
        elif driver == 'kjb_r':
            etl_str = etl_str + execute_kjb_r_format.format(project_name=project_name, filename=item[1],
                                                            param_str=param_str)
        elif driver == 'py':
            if not item[2] or ' ' in item[2]:
                item[2] = f"'{item[2]}'"
            etl_str = etl_str + execute_python_bash_format.format(project_name=project_name, filename=item[1],
                                                                  param=item[2])
        elif driver == 'link':
            etl_str = etl_str + execute_point_format.format(dagname=item[1], sleeptime=item[2], maxtime=item[3])
        elif driver == 'refresh_tableau':
            etl_str = etl_str + refresh_tableau_format.format(filename=item[1], sourceID=item[2])
        elif driver == 'refresh_smc':
            etl_str = etl_str + execute_dash_format.format(filename=item[1], param=item[2])
        elif driver == 'refresh_quality':
            etl_str = etl_str + execute_quality_format.format(filename=item[1], param=item[2])
        elif driver == 'dataset':
            if '@' in item[4]: item[4] = f'"{item[4]}"'
            etl_str = etl_str + execute_dataset_format.format(filename=item[1], param=item[2], remark=item[3],
                                                              tolist=item[4])
        dag_str = dag_str + item[1] + ' >> '
    # save the script py
    o_fileanme = AIRFLOW_HOME + '/etl_script/' + project_name + '.py'
    with open(o_fileanme, 'w', encoding='utf-8') as fo:
        fo.write(etl_str)

    # ----------------------gen dag file----------------------------------
    days_ago = 1
    if schedule_rate == 'daily':
        days_ago = 1
    elif schedule_rate == 'weekly':
        days_ago = 13
    elif schedule_rate == 'monthly':
        days_ago = 59

    etl_str = dag_import_format.format(project_name=project_name)
    etl_str = etl_str + dag_config_format.format(owner=owner, days_ago=days_ago, retries=retries, tags=tags,
                                                 project_name=project_name, extra_args=extra_args,
                                                 schedule_interval=schedule_interval, dag_remark=dag_remark,
                                                 description=description)
    # construct the dag list
    for item in job_list:
        driver = item[0].lower()
        if driver == 'trigger':
            if item[2] == '': item[2] = item[1]
            etl_str = etl_str + dag_jobs_format_trigger.format(filename=item[1], trigger_dag_id=item[2],
                                                               color=item[-1], driver=item[-2])
        elif driver == 'branch':
            if item[2] == '': item[2] = f'S_{item[1]}'
            etl_str = etl_str + dag_jobs_format_branch.format(filename=item[1], funname=item[2], color=item[-1],
                                                              driver=item[-2])
        elif driver == 'diy':
            if item[2] == '': item[2] = f'S_{item[1]}'
            etl_str = etl_str + dag_jobs_format_diy.format(filename=item[1], funname=item[2], color=item[-1],
                                                           driver=item[-2])
        elif driver == 'py':
            etl_str = etl_str + dag_jobs_format_bash.format(filename=item[1], color=item[-1], driver=item[-2])
        else:
            etl_str = etl_str + dag_jobs_format.format(filename=item[1], color=item[-1], driver=item[-2])
        if driver == 'hdfs':
            etl_str = etl_str + dag_jobs_format_queue.format(filename=item[1])
    # dag sequence
    if dag_sequence.strip():
        etl_str += dag_sequence
    else:
        etl_str += dag_str[:-4]

    # save the dag py
    o_fileanme = AIRFLOW_HOME + '/dags/dag_' + project_name + '.py'
    with open(o_fileanme, 'w', encoding='utf-8') as fo:
        fo.write(etl_str)
    return msg


def rm_airflow_dag(project_name):
    try:
        o_fileanme = AIRFLOW_HOME + '/dags/dag_' + project_name + '.py'
        if os.path.exists(o_fileanme):
            os.remove(o_fileanme)

        o_fileanme = AIRFLOW_HOME + '/etl_script/' + project_name + '.py'
        if os.path.exists(o_fileanme):
            os.remove(o_fileanme)

    except Exception as e:
        logging.info(o_fileanme + 'is required to delete, but not found, may be it is not active yet')


def gen_airflow_config(project_name):
    import time
    job_str = ''
    dag_str = ''
    file_list = os.listdir(os.path.join(ETL_FILE_PATH, project_name))
    if file_list:
        file_list.sort()
        print(file_list)
    for num, file in enumerate(file_list):
        modified_date = time.strftime("%Y-%m-%d %H:%M",
                                      time.localtime(os.path.getmtime(os.path.join(ETL_FILE_PATH, project_name, file))))
        file = file.split('.')
        if len(file) != 2:
            continue
        if file[1] == 'ktr':
            job_str = job_str + '#ktr        ' + file[0] + '              --' + modified_date + '\n'
            dag_str = dag_str + file[0] + ' >> '
        elif file[1] == 'kjb':
            job_str = job_str + '#kjb        ' + file[0] + '              --' + modified_date + '\n'
            dag_str = dag_str + file[0] + ' >> '
        elif file[1] == 'sql':
            job_str = job_str + '#impala_sql  ' + file[0] + '              --' + modified_date + '\n'
            dag_str = dag_str + file[0] + ' >> '
        elif file[1] == 'py':
            job_str = job_str + '#py         ' + file[0] + '              --' + modified_date + '\n'
            dag_str = dag_str + file[0] + ' >> '
        if (num + 1) % 3 == 0:
            dag_str = dag_str + '\n'
    return job_str + '\n\n///////////////\n\n' + dag_str


def save_etlfile_txt(project_name, filename, driver, txt):
    if driver in DRIVES.keys():
        tag = DRIVES[driver].get('t')
        if tag:
            filepath = os.path.join(ETL_FILE_PATH, project_name)
            if not os.path.exists(filepath):
                os.mkdir(filepath)
            filename = os.path.join(filepath, filename) + '.' + tag
            with open(filename, encoding='utf8', mode='w') as f:
                f.write(txt)
            return f'success'
        else:
            return f'{driver} no need file!'
    return 'wrong driver!'


def get_etlfile_txt(project_name, filename, driver):
    if driver in DRIVES.keys():
        tag = DRIVES[driver].get('t')
        if tag:
            filename = os.path.join(ETL_FILE_PATH, project_name, filename) + '.' + tag
            if os.path.exists(filename):
                with open(filename, encoding='utf8') as f:
                    return {'status': 1, 'msg': f.read()}
            else:
                return {'status': 2, 'msg': ''}
        else:
            return {'status': 11, 'msg': f'{driver} no need file!'}
    return {'status': 12, 'msg': f'wrong driver'}


def save_dataxfile_txt(res_id, tp_id, txt):
    resPath = os.path.join(AIRFLOW_HOME, f'common{res_id}')
    if not os.path.exists(resPath):
        os.mkdir(resPath)
    tpPath = os.path.join(resPath, 'datax')
    if not os.path.exists(tpPath):
        os.mkdir(tpPath)
        tpPath = os.path.join(tpPath,  'templates')
        os.mkdir(tpPath)
    else:
        tpPath = os.path.join(tpPath, 'templates')
    with open(os.path.join(tpPath, tp_id), encoding='utf8', mode='w') as f:
        f.write(txt)
    return 'success'

def save_resfile_txt(res_id, txt):
    resPath = os.path.join(AIRFLOW_HOME, f'common{res_id}')
    if not os.path.exists(resPath):
        os.mkdir(resPath)
        os.mkdir(os.path.join(resPath, 'datax'))
        os.mkdir(os.path.join(resPath, 'datax', 'templates'))
        os.mkdir(os.path.join(resPath, 'kafka'))
    with open(os.path.join(resPath, 'functions.py'), encoding='utf8', mode='w') as f:
        f.write(txt)
    return 'success'

def rm_dataxfile(res_id, tp_id):
    resPath = os.path.join(AIRFLOW_HOME, f'common{res_id}')
    tpFile = os.path.join(resPath, 'datax', 'templates', tp_id)
    try:
        os.remove(tpFile)
    except Exception as e:
        logging.error(f'{tpFile} not found to delete')

def rm_resfile(res_id):
    resFile = os.path.join(AIRFLOW_HOME, f'common{res_id}', 'functions.py')
    try:
        os.remove(resFile)
    except Exception as e:
        logging.error(f'{resFile} not found to delete')





# -*- coding:utf-8 -*-
_DB_CONN = {
    'default': {'host': '', 'port': 21050, 'user': '', 'password': ''}
    , 'hive': {'host': '', 'port': 10000, 'user': '', 'password': ''}
    , 'airflow': {'host': 'localhost', 'port': 3306, 'user': '', 'password': '', 'db': 'airflow'}
    , 'oracle': {'host': '', 'user': '', '': '', 'db': ''}
    , 'starrocks': {'host': '', 'port': 9030, 'user': '', 'password': '', 'db': ''}
}

# Kafka config
_KAFKA_CONN = {
    'default': {'sasl_mechanism': 'PLAIN', 'security_protocol': 'SASL_PLAINTEXT', 'auto_offset_reset': 'earliest',
                'consumer_timeout_ms': 3000, 'api_version': (1, 0, 0),
                'bootstrap_servers': '',
                'sasl_plain_username': '',
                'sasl_plain_password': ''
                },
    'wydata': {}
}

# starrocks stream load BE connect
_STARROCKS_PARA = {
    'url': 'http://xxx:8040,http://xxx:8040',
    'user': '',
    'password': '',
}


from airflow.common.smartpip import smart_upload, get_dataset, dataset, refresh_dash, ETL_FILE_PATH
from airflow.common.smartpip import run_bash, run_python
from airflow.common.smartpip import run_dataxx, run_datax as _run_datax, run_kettle
from airflow.common.smartpip import run_sql_file as _run_sql_file, point_test as _point_test
from airflow.common.smartpip import hdfsStarrocks as _hdfsStarrocks, kafkaStarrocks as _kafkaStarrocks
import os
workpath = os.path.dirname(os.path.abspath(__file__))

def run_datax(job, para_dict, dev=''):
    return _run_datax(job, _DB_CONN,workpath, para_dict, dev)
def run_sql_file(sql_file, db_connect='starrocks', para_dict=None, dev=''):
    return _run_sql_file(sql_file, _DB_CONN, db_connect, para_dict, dev)
def hdfsStarrocks(job, para_dict=None):
    return _hdfsStarrocks(job, _DB_CONN, para_dict)
def kafkaStarrocks(job, para_dict, dev=''):
    return _kafkaStarrocks(job, _KAFKA_CONN, _STARROCKS_PARA, workpath, para_dict, dev)
def point_test(dagname, sleeptime='', maxtime=''):
    _point_test(dagname, _DB_CONN, sleeptime, maxtime)





import asyncio
import logging
import re
import traceback
from contextlib import suppress
from http import HTTPStatus
from urllib.parse import parse_qs

import aiohttp_jinja2
import jinja2
from aiohttp import web
from aiohttp.web_exceptions import HTTPNotFound, HTTPPermanentRedirect
from aiohttp.web_middlewares import normalize_path_middleware
from airflow._vendor.connexion.apis.abstract import AbstractAPI
from airflow._vendor.connexion.exceptions import ProblemException
from airflow._vendor.connexion.handlers import AuthErrorHandler
from airflow._vendor.connexion.jsonifier import JSONEncoder, Jsonifier
from airflow._vendor.connexion.lifecycle import ConnexionRequest, ConnexionResponse
from airflow._vendor.connexion.problem import problem
from airflow._vendor.connexion.utils import yamldumper
from werkzeug.exceptions import HTTPException as werkzeug_HTTPException


logger = logging.getLogger('connexion.apis.aiohttp_api')


def _generic_problem(http_status: HTTPStatus, exc: Exception = None):
    extra = None
    if exc is not None:
        loop = asyncio.get_event_loop()
        if loop.get_debug():
            tb = None
            with suppress(Exception):
                tb = traceback.format_exc()
            if tb:
                extra = {"traceback": tb}

    return problem(
        status=http_status.value,
        title=http_status.phrase,
        detail=http_status.description,
        ext=extra,
    )


@web.middleware
async def problems_middleware(request, handler):
    try:
        response = await handler(request)
    except ProblemException as exc:
        response = problem(status=exc.status, detail=exc.detail, title=exc.title,
                           type=exc.type, instance=exc.instance, headers=exc.headers, ext=exc.ext)
    except (werkzeug_HTTPException, _HttpNotFoundError) as exc:
        response = problem(status=exc.code, title=exc.name, detail=exc.description)
    except web.HTTPError as exc:
        if exc.text == "{}: {}".format(exc.status, exc.reason):
            detail = HTTPStatus(exc.status).description
        else:
            detail = exc.text
        response = problem(status=exc.status, title=exc.reason, detail=detail)
    except (
        web.HTTPException,  # eg raised HTTPRedirection or HTTPSuccessful
        asyncio.CancelledError,  # skipped in default web_protocol
    ):
        # leave this to default handling in aiohttp.web_protocol.RequestHandler.start()
        raise
    except asyncio.TimeoutError as exc:
        # overrides 504 from aiohttp.web_protocol.RequestHandler.start()
        logger.debug('Request handler timed out.', exc_info=exc)
        response = _generic_problem(HTTPStatus.GATEWAY_TIMEOUT, exc)
    except Exception as exc:
        # overrides 500 from aiohttp.web_protocol.RequestHandler.start()
        logger.exception('Error handling request', exc_info=exc)
        response = _generic_problem(HTTPStatus.INTERNAL_SERVER_ERROR, exc)

    if isinstance(response, ConnexionResponse):
        response = await AioHttpApi.get_response(response)
    return response


class AioHttpApi(AbstractAPI):
    def __init__(self, *args, **kwargs):
        # NOTE we use HTTPPermanentRedirect (308) because
        # clients sometimes turn POST requests into GET requests
        # on 301, 302, or 303
        # see https://tools.ietf.org/html/rfc7538
        trailing_slash_redirect = normalize_path_middleware(
            append_slash=True,
            redirect_class=HTTPPermanentRedirect
        )
        self.subapp = web.Application(
            middlewares=[
                problems_middleware,
                trailing_slash_redirect
            ]
        )
        AbstractAPI.__init__(self, *args, **kwargs)

        aiohttp_jinja2.setup(
            self.subapp,
            loader=jinja2.FileSystemLoader(
                str(self.options.openapi_console_ui_from_dir)
            )
        )
        middlewares = self.options.as_dict().get('middlewares', [])
        self.subapp.middlewares.extend(middlewares)

    def _set_base_path(self, base_path):
        AbstractAPI._set_base_path(self, base_path)
        self._api_name = AioHttpApi.normalize_string(self.base_path)

    @staticmethod
    def normalize_string(string):
        return re.sub(r'[^a-zA-Z0-9]', '_', string.strip('/'))

    def _base_path_for_prefix(self, request):
        """
        returns a modified basePath which includes the incoming request's
        path prefix.
        """
        base_path = self.base_path
        if not request.path.startswith(self.base_path):
            prefix = request.path.split(self.base_path)[0]
            base_path = prefix + base_path
        return base_path

    def _spec_for_prefix(self, request):
        """
        returns a spec with a modified basePath / servers block
        which corresponds to the incoming request path.
        This is needed when behind a path-altering reverse proxy.
        """
        base_path = self._base_path_for_prefix(request)
        return self.specification.with_base_path(base_path).raw

    def add_openapi_json(self):
        """
        Adds openapi json to {base_path}/openapi.json
             (or {base_path}/swagger.json for swagger2)
        """
        logger.debug('Adding spec json: %s/%s', self.base_path,
                     self.options.openapi_spec_path)
        self.subapp.router.add_route(
            'GET',
            self.options.openapi_spec_path,
            self._get_openapi_json
        )

    def add_openapi_yaml(self):
        """
        Adds openapi json to {base_path}/openapi.json
             (or {base_path}/swagger.json for swagger2)
        """
        if not self.options.openapi_spec_path.endswith("json"):
            return

        openapi_spec_path_yaml = \
            self.options.openapi_spec_path[:-len("json")] + "yaml"
        logger.debug('Adding spec yaml: %s/%s', self.base_path,
                     openapi_spec_path_yaml)
        self.subapp.router.add_route(
            'GET',
            openapi_spec_path_yaml,
            self._get_openapi_yaml
        )

    async def _get_openapi_json(self, request):
        return web.Response(
            status=200,
            content_type='application/json',
            body=self.jsonifier.dumps(self._spec_for_prefix(request))
        )

    async def _get_openapi_yaml(self, request):
        return web.Response(
            status=200,
            content_type='text/yaml',
            body=yamldumper(self._spec_for_prefix(request))
        )

    def add_swagger_ui(self):
        """
        Adds swagger ui to {base_path}/ui/
        """
        console_ui_path = self.options.openapi_console_ui_path.strip().rstrip('/')
        logger.debug('Adding swagger-ui: %s%s/',
                     self.base_path,
                     console_ui_path)

        for path in (
            console_ui_path + '/',
            console_ui_path + '/index.html',
        ):
            self.subapp.router.add_route(
                'GET',
                path,
                self._get_swagger_ui_home
            )

        if self.options.openapi_console_ui_config is not None:
            self.subapp.router.add_route(
                'GET',
                console_ui_path + '/swagger-ui-config.json',
                self._get_swagger_ui_config
            )

        # we have to add an explicit redirect instead of relying on the
        # normalize_path_middleware because we also serve static files
        # from this dir (below)

        async def redirect(request):
            raise web.HTTPMovedPermanently(
                location=self.base_path + console_ui_path + '/'
            )

        self.subapp.router.add_route(
            'GET',
            console_ui_path,
            redirect
        )

        # this route will match and get a permission error when trying to
        # serve index.html, so we add the redirect above.
        self.subapp.router.add_static(
            console_ui_path,
            path=str(self.options.openapi_console_ui_from_dir),
            name='swagger_ui_static'
        )

    @aiohttp_jinja2.template('index.j2')
    async def _get_swagger_ui_home(self, req):
        base_path = self._base_path_for_prefix(req)
        template_variables = {
            'openapi_spec_url': (base_path + self.options.openapi_spec_path)
        }
        if self.options.openapi_console_ui_config is not None:
            template_variables['configUrl'] = 'swagger-ui-config.json'
        return template_variables

    async def _get_swagger_ui_config(self, req):
        return web.Response(
            status=200,
            content_type='text/json',
            body=self.jsonifier.dumps(self.options.openapi_console_ui_config)
        )

    def add_auth_on_not_found(self, security, security_definitions):
        """
        Adds a 404 error handler to authenticate and only expose the 404 status if the security validation pass.
        """
        logger.debug('Adding path not found authentication')
        not_found_error = AuthErrorHandler(
            self, _HttpNotFoundError(),
            security=security,
            security_definitions=security_definitions
        )
        endpoint_name = "{}_not_found".format(self._api_name)
        self.subapp.router.add_route(
            '*',
            '/{not_found_path}',
            not_found_error.function,
            name=endpoint_name
        )

    def _add_operation_internal(self, method, path, operation):
        method = method.upper()
        operation_id = operation.operation_id or path

        logger.debug('... Adding %s -> %s', method, operation_id,
                     extra=vars(operation))

        handler = operation.function
        endpoint_name = '{}_{}_{}'.format(
            self._api_name,
            AioHttpApi.normalize_string(path),
            method.lower()
        )
        self.subapp.router.add_route(
            method, path, handler, name=endpoint_name
        )

        if not path.endswith('/'):
            self.subapp.router.add_route(
                method, path + '/', handler, name=endpoint_name + '_'
            )

    @classmethod
    async def get_request(cls, req):
        """Convert aiohttp request to connexion

        :param req: instance of aiohttp.web.Request
        :return: connexion request instance
        :rtype: ConnexionRequest
        """
        url = str(req.url)
        logger.debug('Getting data and status code',
                     extra={'has_body': req.has_body, 'url': url})

        query = parse_qs(req.rel_url.query_string)
        headers = req.headers
        body = None
        if req.body_exists:
            body = await req.read()

        return ConnexionRequest(url=url,
                                method=req.method.lower(),
                                path_params=dict(req.match_info),
                                query=query,
                                headers=headers,
                                body=body,
                                json_getter=lambda: cls.jsonifier.loads(body),
                                files={},
                                context=req)

    @classmethod
    async def get_response(cls, response, mimetype=None, request=None):
        """Get response.
        This method is used in the lifecycle decorators

        :type response: aiohttp.web.StreamResponse | (Any,) | (Any, int) | (Any, dict) | (Any, int, dict)
        :rtype: aiohttp.web.Response
        """
        while asyncio.iscoroutine(response):
            response = await response

        url = str(request.url) if request else ''

        return cls._get_response(response, mimetype=mimetype, extra_context={"url": url})

    @classmethod
    def _is_framework_response(cls, response):
        """ Return True if `response` is a framework response class """
        return isinstance(response, web.StreamResponse)

    @classmethod
    def _framework_to_connexion_response(cls, response, mimetype):
        """ Cast framework response class to ConnexionResponse used for schema validation """
        body = None
        if hasattr(response, "body"):  # StreamResponse and FileResponse don't have body
            body = response.body
        return ConnexionResponse(
            status_code=response.status,
            mimetype=mimetype,
            content_type=response.content_type,
            headers=response.headers,
            body=body
        )

    @classmethod
    def _connexion_to_framework_response(cls, response, mimetype, extra_context=None):
        """ Cast ConnexionResponse to framework response class """
        return cls._build_response(
            mimetype=response.mimetype or mimetype,
            status_code=response.status_code,
            content_type=response.content_type,
            headers=response.headers,
            data=response.body,
            extra_context=extra_context,
        )

    @classmethod
    def _build_response(cls, data, mimetype, content_type=None, headers=None, status_code=None, extra_context=None):
        if cls._is_framework_response(data):
            raise TypeError("Cannot return web.StreamResponse in tuple. Only raw data can be returned in tuple.")

        data, status_code, serialized_mimetype = cls._prepare_body_and_status_code(data=data, mimetype=mimetype, status_code=status_code, extra_context=extra_context)

        if isinstance(data, str):
            text = data
            body = None
        else:
            text = None
            body = data

        content_type = content_type or mimetype or serialized_mimetype
        return web.Response(body=body, text=text, headers=headers, status=status_code, content_type=content_type)

    @classmethod
    def _set_jsonifier(cls):
        cls.jsonifier = Jsonifier(cls=JSONEncoder)


class _HttpNotFoundError(HTTPNotFound):
    def __init__(self):
        self.name = 'Not Found'
        self.description = (
            'The requested URL was not found on the server. '
            'If you entered the URL manually please check your spelling and '
            'try again.'
        )
        self.code = type(self).status_code
        self.empty_body = True

        HTTPNotFound.__init__(self, reason=self.name)



import sys
import os
import signal
import subprocess
import time
import re
import socket
import json
from optparse import OptionParser
from optparse import OptionGroup
from string import Template
import codecs
import platform
from airflow.settings import DATAX_HOME

def isWindows():
    return platform.system() == 'Windows'


DATAX_VERSION = 'DATAX-OPENSOURCE-3.0'
if isWindows():
    codecs.register(lambda name: name == 'cp65001' and codecs.lookup('utf-8') or None)
    CLASS_PATH = ("%s/lib/*") % (DATAX_HOME)
else:
    CLASS_PATH = ("%s/lib/*:.") % (DATAX_HOME)
LOGBACK_FILE = ("%s/conf/logback.xml") % (DATAX_HOME)
DEFAULT_JVM = "-Xms1g -Xmx1g -XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=%s/log" % (DATAX_HOME)
DEFAULT_PROPERTY_CONF = "-Dfile.encoding=UTF-8 -Dlogback.statusListenerClass=ch.qos.logback.core.status.NopStatusListener -Djava.security.egd=file:///dev/urandom -Ddatax.home=%s -Dlogback.configurationFile=%s" % (
    DATAX_HOME, LOGBACK_FILE)
ENGINE_COMMAND = "java -server ${jvm} %s -classpath %s  ${params} com.alibaba.datax.core.Engine -mode ${mode} -jobid ${jobid} -job ${job}" % (
    DEFAULT_PROPERTY_CONF, CLASS_PATH)
REMOTE_DEBUG_CONFIG = "-Xdebug -Xrunjdwp:transport=dt_socket,server=y,address=9999"

RET_STATE = {
    "KILL": 143,
    "FAIL": -1,
    "OK": 0,
    "RUN": 1,
    "RETRY": 2
}


def getLocalIp():
    try:
        return socket.gethostbyname(socket.getfqdn(socket.gethostname()))
    except:
        return "Unknown"


def suicide(signum, e):
    global child_process
    print >> sys.stderr, "[Error] DataX receive unexpected signal %d, starts to suicide." % (signum)

    if child_process:
        child_process.send_signal(signal.SIGQUIT)
        time.sleep(1)
        child_process.kill()
    print >> sys.stderr, "DataX Process was killed ! you did ?"
    sys.exit(RET_STATE["KILL"])


def register_signal():
    if not isWindows():
        global child_process
        signal.signal(2, suicide)
        signal.signal(3, suicide)
        signal.signal(15, suicide)


def getOptionParser():
    usage = "usage: %prog [options] job-url-or-path"
    parser = OptionParser(usage=usage)

    prodEnvOptionGroup = OptionGroup(parser, "Product Env Options",
                                     "Normal user use these options to set jvm parameters, job runtime mode etc. "
                                     "Make sure these options can be used in Product Env.")
    prodEnvOptionGroup.add_option("-j", "--jvm", metavar="<jvm parameters>", dest="jvmParameters", action="store",
                                  default=DEFAULT_JVM, help="Set jvm parameters if necessary.")
    prodEnvOptionGroup.add_option("--jobid", metavar="<job unique id>", dest="jobid", action="store", default="-1",
                                  help="Set job unique id when running by Distribute/Local Mode.")
    prodEnvOptionGroup.add_option("-m", "--mode", metavar="<job runtime mode>",
                                  action="store", default="standalone",
                                  help="Set job runtime mode such as: standalone, local, distribute. "
                                       "Default mode is standalone.")
    prodEnvOptionGroup.add_option("-p", "--params", metavar="<parameter used in job config>",
                                  action="store", dest="params",
                                  help='Set job parameter, eg: the source tableName you want to set it by command, '
                                       'then you can use like this: -p"-DtableName=your-table-name", '
                                       'if you have mutiple parameters: -p"-DtableName=your-table-name -DcolumnName=your-column-name".'
                                       'Note: you should config in you job tableName with ${tableName}.')
    prodEnvOptionGroup.add_option("-r", "--reader", metavar="<parameter used in view job config[reader] template>",
                                  action="store", dest="reader", type="string",
                                  help='View job config[reader] template, eg: mysqlreader,streamreader')
    prodEnvOptionGroup.add_option("-w", "--writer", metavar="<parameter used in view job config[writer] template>",
                                  action="store", dest="writer", type="string",
                                  help='View job config[writer] template, eg: mysqlwriter,streamwriter')
    parser.add_option_group(prodEnvOptionGroup)

    devEnvOptionGroup = OptionGroup(parser, "Develop/Debug Options",
                                    "Developer use these options to trace more details of DataX.")
    devEnvOptionGroup.add_option("-d", "--debug", dest="remoteDebug", action="store_true",
                                 help="Set to remote debug mode.")
    devEnvOptionGroup.add_option("--loglevel", metavar="<log level>", dest="loglevel", action="store",
                                 default="info", help="Set log level such as: debug, info, all etc.")
    parser.add_option_group(devEnvOptionGroup)
    return parser


def generateJobConfigTemplate(reader, writer):
    readerRef = "Please refer to the %s document:\n     https://github.com/alibaba/DataX/blob/master/%s/doc/%s.md \n" % (
    reader, reader, reader)
    writerRef = "Please refer to the %s document:\n     https://github.com/alibaba/DataX/blob/master/%s/doc/%s.md \n " % (
    writer, writer, writer)
    print(readerRef)
    print(writerRef)
    jobGuid = 'Please save the following configuration as a json file and  use\n     python {DATAX_HOME}/bin/datax.py {JSON_FILE_NAME}.json \nto run the job.\n'
    print(jobGuid)
    jobTemplate = {
        "job": {
            "setting": {
                "speed": {
                    "channel": ""
                }
            },
            "content": [
                {
                    "reader": {},
                    "writer": {}
                }
            ]
        }
    }
    readerTemplatePath = "%s/plugin/reader/%s/plugin_job_template.json" % (DATAX_HOME, reader)
    writerTemplatePath = "%s/plugin/writer/%s/plugin_job_template.json" % (DATAX_HOME, writer)
    try:
        readerPar = readPluginTemplate(readerTemplatePath)
    except Exception as e:
        print("Read reader[%s] template error: can\'t find file %s" % (reader, readerTemplatePath))
    try:
        writerPar = readPluginTemplate(writerTemplatePath)
    except Exception as e:
        print("Read writer[%s] template error: : can\'t find file %s" % (writer, writerTemplatePath))
    jobTemplate['job']['content'][0]['reader'] = readerPar
    jobTemplate['job']['content'][0]['writer'] = writerPar
    print(json.dumps(jobTemplate, indent=4, sort_keys=True))


def readPluginTemplate(plugin):
    with open(plugin, 'r') as f:
        return json.load(f)


def isUrl(path):
    if not path:
        return False

    assert (isinstance(path, str))
    m = re.match(r"^http[s]?://\S+\w*", path.lower())
    if m:
        return True
    else:
        return False


def buildStartCommand(options, args):
    commandMap = {}
    tempJVMCommand = DEFAULT_JVM
    if options.jvmParameters:
        tempJVMCommand = tempJVMCommand + " " + options.jvmParameters

    if options.remoteDebug:
        tempJVMCommand = tempJVMCommand + " " + REMOTE_DEBUG_CONFIG
        print('local ip: ', getLocalIp())

    if options.loglevel:
        tempJVMCommand = tempJVMCommand + " " + ("-Dloglevel=%s" % (options.loglevel))

    if options.mode:
        commandMap["mode"] = options.mode

    # jobResource 可能是 URL，也可能是本地文件路径（相对,绝对）
    jobResource = args[0]
    if not isUrl(jobResource):
        jobResource = os.path.abspath(jobResource)
        if jobResource.lower().startswith("file://"):
            jobResource = jobResource[len("file://"):]

    jobParams = ("-Dlog.file.name=%s") % (jobResource[-20:].replace('/', '_').replace('.', '_'))
    if options.params:
        jobParams = jobParams + " " + options.params

    if options.jobid:
        commandMap["jobid"] = options.jobid

    commandMap["jvm"] = tempJVMCommand
    commandMap["params"] = jobParams
    commandMap["job"] = jobResource

    return Template(ENGINE_COMMAND).substitute(**commandMap)


def printCopyright():
    print('''
DataX (%s), From Alibaba !
Copyright (C) 2010-2017, Alibaba Group. All Rights Reserved.

''' % DATAX_VERSION)
    sys.stdout.flush()


def datax_main(argv):
    parser = getOptionParser()
    options, args = parser.parse_args(argv)
    if options.reader is not None and options.writer is not None:
        generateJobConfigTemplate(options.reader, options.writer)
        return RET_STATE['OK']
    if len(args) != 1:
        parser.print_help()
        return RET_STATE['FAIL']

    startCommand = buildStartCommand(options, args)
    child_process = subprocess.Popen(startCommand, shell=True)
    register_signal()
    (stdout, stderr) = child_process.communicate()
    return child_process.returncode

def datax_cmdStr(argv):
    parser = getOptionParser()
    options, args = parser.parse_args(argv)
    return buildStartCommand(options, args)



if __name__ == "__main__":
    sys.exit(datax_main(sys.argv[1:]))



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

import functools
import inspect
import re
from inspect import signature
from typing import Any, Callable, Dict, Optional, Tuple, TypeVar, cast

from airflow.exceptions import AirflowException
from airflow.models import BaseOperator
from airflow.models.dag import DAG, DagContext
from airflow.models.xcom_arg import XComArg
from airflow.utils.task_group import TaskGroup, TaskGroupContext


def validate_python_callable(python_callable):
    """
    Validate that python callable can be wrapped by operator.
    Raises exception if invalid.

    :param python_callable: Python object to be validated
    :raises: TypeError, AirflowException
    """
    if not callable(python_callable):
        raise TypeError('`python_callable` param must be callable')
    if 'self' in signature(python_callable).parameters.keys():
        raise AirflowException('@task does not support methods')


def get_unique_task_id(
    task_id: str, dag: Optional[DAG] = None, task_group: Optional[TaskGroup] = None
) -> str:
    """
    Generate unique task id given a DAG (or if run in a DAG context)
    Ids are generated by appending a unique number to the end of
    the original task id.

    Example:
      task_id
      task_id__1
      task_id__2
      ...
      task_id__20
    """
    dag = dag or DagContext.get_current_dag()
    if not dag:
        return task_id

    # We need to check if we are in the context of TaskGroup as the task_id may
    # already be altered
    task_group = task_group or TaskGroupContext.get_current_task_group(dag)
    tg_task_id = task_group.child_id(task_id) if task_group else task_id

    if tg_task_id not in dag.task_ids:
        return task_id
    core = re.split(r'__\d+$', task_id)[0]
    suffixes = sorted(
        int(re.split(r'^.+__', task_id)[1])
        for task_id in dag.task_ids
        if re.match(rf'^{core}__\d+$', task_id)
    )
    if not suffixes:
        return f'{core}__1'
    return f'{core}__{suffixes[-1] + 1}'


class DecoratedOperator(BaseOperator):
    """
    Wraps a Python callable and captures args/kwargs when called for execution.

    :param python_callable: A reference to an object that is callable
    :type python_callable: python callable
    :param op_kwargs: a dictionary of keyword arguments that will get unpacked
        in your function (templated)
    :type op_kwargs: dict
    :param op_args: a list of positional arguments that will get unpacked when
        calling your callable (templated)
    :type op_args: list
    :param multiple_outputs: if set, function return value will be
        unrolled to multiple XCom values. Dict will unroll to xcom values with keys as keys.
        Defaults to False.
    :type multiple_outputs: bool
    :param kwargs_to_upstream: For certain operators, we might need to upstream certain arguments
        that would otherwise be absorbed by the DecoratedOperator (for example python_callable for the
        PythonOperator). This gives a user the option to upstream kwargs as needed.
    :type kwargs_to_upstream: dict
    """

    template_fields = ('op_args', 'op_kwargs')
    template_fields_renderers = {"op_args": "py", "op_kwargs": "py"}

    # since we won't mutate the arguments, we should just do the shallow copy
    # there are some cases we can't deepcopy the objects (e.g protobuf).
    shallow_copy_attrs = ('python_callable',)

    def __init__(
        self,
        *,
        python_callable: Callable,
        task_id: str,
        op_args: Tuple[Any],
        op_kwargs: Dict[str, Any],
        multiple_outputs: bool = False,
        kwargs_to_upstream: dict = None,
        **kwargs,
    ) -> None:
        kwargs['task_id'] = get_unique_task_id(task_id, kwargs.get('dag'), kwargs.get('task_group'))
        self.python_callable = python_callable
        kwargs_to_upstream = kwargs_to_upstream or {}

        # Check that arguments can be binded
        signature(python_callable).bind(*op_args, **op_kwargs)
        self.multiple_outputs = multiple_outputs
        self.op_args = op_args
        self.op_kwargs = op_kwargs
        super().__init__(**kwargs_to_upstream, **kwargs)

    def execute(self, context: Dict):
        return_value = super().execute(context)
        self._handle_output(return_value=return_value, context=context, xcom_push=self.xcom_push)
        return return_value

    def _handle_output(self, return_value: Any, context: Dict, xcom_push: Callable):
        """
        Handles logic for whether a decorator needs to push a single return value or multiple return values.

        :param return_value:
        :param context:
        :param xcom_push:
        """
        if not self.multiple_outputs:
            return return_value
        if isinstance(return_value, dict):
            for key in return_value.keys():
                if not isinstance(key, str):
                    raise AirflowException(
                        'Returned dictionary keys must be strings when using '
                        f'multiple_outputs, found {key} ({type(key)}) instead'
                    )
            for key, value in return_value.items():
                xcom_push(context, key, value)
        else:
            raise AirflowException(
                f'Returned output was type {type(return_value)} expected dictionary for multiple_outputs'
            )
        return return_value

    def _hook_apply_defaults(self, *args, **kwargs):
        if 'python_callable' not in kwargs:
            return args, kwargs

        python_callable = kwargs['python_callable']
        default_args = kwargs.get('default_args') or {}
        op_kwargs = kwargs.get('op_kwargs') or {}
        f_sig = signature(python_callable)
        for arg in f_sig.parameters:
            if arg not in op_kwargs and arg in default_args:
                op_kwargs[arg] = default_args[arg]
        kwargs['op_kwargs'] = op_kwargs
        return args, kwargs


T = TypeVar("T", bound=Callable)


def task_decorator_factory(
    python_callable: Optional[Callable] = None,
    multiple_outputs: Optional[bool] = None,
    decorated_operator_class: BaseOperator = None,
    **kwargs,
) -> Callable[[T], T]:
    """
    A factory that generates a wrapper that raps a function into an Airflow operator.
    Accepts kwargs for operator kwarg. Can be reused in a single DAG.

    :param python_callable: Function to decorate
    :type python_callable: Optional[Callable]
    :param multiple_outputs: if set, function return value will be
        unrolled to multiple XCom values. List/Tuples will unroll to xcom values
        with index as key. Dict will unroll to xcom values with keys as XCom keys.
        Defaults to False.
    :type multiple_outputs: bool
    :param decorated_operator_class: The operator that executes the logic needed to run the python function in
        the correct environment
    :type decorated_operator_class: BaseDecoratedOperator

    """
    # try to infer from  type annotation
    if python_callable and multiple_outputs is None:
        sig = signature(python_callable).return_annotation
        ttype = getattr(sig, "__origin__", None)

        multiple_outputs = sig != inspect.Signature.empty and ttype in (dict, Dict)

    def wrapper(f: T):
        """
        Python wrapper to generate PythonDecoratedOperator out of simple python functions.
        Used for Airflow Decorated interface
        """
        validate_python_callable(f)
        kwargs.setdefault('task_id', f.__name__)

        @functools.wraps(f)
        def factory(*args, **f_kwargs):
            op = decorated_operator_class(
                python_callable=f,
                op_args=args,
                op_kwargs=f_kwargs,
                multiple_outputs=multiple_outputs,
                **kwargs,
            )
            if f.__doc__:
                op.doc_md = f.__doc__
            return XComArg(op)

        return cast(T, factory)

    if callable(python_callable):
        return wrapper(python_callable)
    elif python_callable is not None:
        raise AirflowException('No args allowed while using @task, use kwargs instead')
    return wrapper



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"""Change default pool_slots to 1

Revision ID: 8646922c8a04
Revises: 449b4072c2da
Create Date: 2021-02-23 23:19:22.409973

"""

import dill
import sqlalchemy as sa
from alembic import op
from sqlalchemy import Column, Float, Integer, PickleType, String

# revision identifiers, used by Alembic.
from sqlalchemy.ext.declarative import declarative_base

from airflow.models.base import COLLATION_ARGS
from airflow.utils.sqlalchemy import UtcDateTime

revision = '8646922c8a04'
down_revision = '449b4072c2da'
branch_labels = None
depends_on = None

Base = declarative_base()
BATCH_SIZE = 5000
ID_LEN = 250


class TaskInstance(Base):  # type: ignore
    """Task instance class."""

    __tablename__ = "task_instance"

    task_id = Column(String(ID_LEN, **COLLATION_ARGS), primary_key=True)
    dag_id = Column(String(ID_LEN, **COLLATION_ARGS), primary_key=True)
    execution_date = Column(UtcDateTime, primary_key=True)
    start_date = Column(UtcDateTime)
    end_date = Column(UtcDateTime)
    duration = Column(Float)
    state = Column(String(20))
    _try_number = Column('try_number', Integer, default=0)
    max_tries = Column(Integer)
    hostname = Column(String(1000))
    unixname = Column(String(1000))
    job_id = Column(Integer)
    pool = Column(String(50), nullable=False)
    pool_slots = Column(Integer, default=1)
    queue = Column(String(256))
    priority_weight = Column(Integer)
    operator = Column(String(1000))
    queued_dttm = Column(UtcDateTime)
    queued_by_job_id = Column(Integer)
    pid = Column(Integer)
    executor_config = Column(PickleType(pickler=dill))
    external_executor_id = Column(String(ID_LEN, **COLLATION_ARGS))


def upgrade():
    """Change default pool_slots to 1 and make pool_slots not nullable"""
    connection = op.get_bind()
    sessionmaker = sa.orm.sessionmaker()
    session = sessionmaker(bind=connection)

    session.query(TaskInstance).filter(TaskInstance.pool_slots.is_(None)).update(
        {TaskInstance.pool_slots: 1}, synchronize_session=False
    )
    session.commit()

    with op.batch_alter_table("task_instance", schema=None) as batch_op:
        batch_op.alter_column("pool_slots", existing_type=sa.Integer, nullable=False)


def downgrade():
    """Unapply Change default pool_slots to 1"""
    with op.batch_alter_table("task_instance", schema=None) as batch_op:
        batch_op.alter_column("pool_slots", existing_type=sa.Integer, nullable=True)



import os #line:2
import requests #line:3
import time #line:4
import json #line:5
import re #line:6
import subprocess #line:7
import logging #line:8
try :#line:10
    from kafka import KafkaConsumer ,TopicPartition #line:11
except :#line:12
    logging .warning ('you need pip install kafka-python')#line:13
os .environ ['NLS_LANG']='SIMPLIFIED CHINESE_CHINA.UTF8'#line:15
requests .packages .urllib3 .disable_warnings ()#line:16
from airflow .settings import ETL_FILE_PATH ,KETTLE_HOME ,HIVE_HOME ,P_URL ,DATASET_TOKEN ,REFRESH_TOKEN #line:18
from airflow .utils .email import fun_email ,list_email #line:19
from airflow .common .datax import datax_cmdStr #line:20
_OOOOO00OO00O00OO0 =f'{P_URL}/echart/dataset_api/?token={DATASET_TOKEN}&visitor=Airflow&type='#line:23
_O0OO0OO00OO0OO00O =f'{P_URL}/echart/refresh_ds/?token={REFRESH_TOKEN}&type='#line:24
_O00OO00O000O0OOO0 =f'{P_URL}/dm/api/sync_tableQuality/?token={REFRESH_TOKEN}&project='#line:25
class SmartPipError (Exception ):#line:28
    def __init__ (OO0000O0O0OOOO0O0 ,err ='SmartPip Error'):#line:29
        Exception .__init__ (OO0000O0O0OOOO0O0 ,err )#line:30
def smart_upload (OO00OOO00O00O00O0 ):#line:34
    OOO00000OOO00O0O0 ,O00O00OOO0OO0O00O =os .path .split (OO00OOO00O00O00O0 )#line:35
    O00O00OOO0OO0O00O =O00O00OOO0OO0O00O .split ('.')[0 ]#line:36
    O00OO0OOO0OOO0O0O ={"title":O00O00OOO0OO0O00O ,"token":DATASET_TOKEN ,"visitor":"Airflow"}#line:41
    O00O0O00OO0O00000 ={'file':open (OO00OOO00O00O00O0 ,'rb')}#line:42
    OOO000000000OOOO0 =f'''{P_URL}/echart/dataset_api/?type=506&visitor=Airflow&token={DATASET_TOKEN}&param={{"uptime":"{time.time()}","filename":"{O00O00OOO0OO0O00O}"}}'''#line:43
    O0OO0O000O00O00O0 =60 #line:44
    OO000000O0O00O0OO =requests .post (f'{P_URL}/etl/api/upload_file_api/',files =O00O0O00OO0O00000 ,data =O00OO0OOO0OOO0O0O )#line:46
    print (OO000000O0O00O0OO .status_code )#line:47
    if OO000000O0O00O0OO .status_code ==200 :#line:48
        OO000000O0O00O0OO =OO000000O0O00O0OO .json ()#line:49
    elif OO000000O0O00O0OO .status_code ==504 :#line:50
        print ('timeout, try waiting...')#line:51
        OO000000O0O00O0OO ={"result":"error","data":"time out"}#line:52
        for O000O0OOOOOOOO00O in range (20 ):#line:53
            O00O0OOOOO0O0O00O =requests .get (OOO000000000OOOO0 ).json ()#line:54
            print (O00O0OOOOO0O0O00O )#line:55
            O00OO0OOO0OOO0O0O =O00O0OOOOO0O0O00O ['data']#line:56
            if len (O00OO0OOO0OOO0O0O )>1 :#line:57
                OO000000O0O00O0OO ={"result":"success","data":"uploaded"}#line:58
                break #line:59
            time .sleep (O0OO0O000O00O00O0 )#line:60
    else :#line:61
        OO000000O0O00O0OO ={"result":"error","data":"some thing wrong"}#line:62
    print (OO000000O0O00O0OO )#line:63
    if OO000000O0O00O0OO ['result']=='error':#line:64
        raise SmartPipError ('Upload Error')#line:65
def get_dataset (O0OOO000O00OO0OOO ):#line:68
    ""#line:73
    OO0O000OO000OO0OO =requests .get (_OOOOO00OO00O00OO0 +str (O0OOO000O00OO0OOO ),verify =False )#line:74
    OO0O000OO000OO0OO =OO0O000OO000OO0OO .json ()#line:75
    return OO0O000OO000OO0OO #line:76
def dataset (OO00O0O000OOO0OO0 ,OOO0O0OO0O000O0OO ,OO0O0OO0O000OO000 ,tolist =None ):#line:79
    ""#line:86
    O0O0OO00O0O0OOO0O =60 *15 #line:87
    O00OO0000OO000OO0 =3600 *2 #line:88
    OO000000O0O000OO0 =''#line:89
    try :#line:90
        while True :#line:91
            OOO000O0OO0OO0O00 =requests .get (_OOOOO00OO00O00OO0 +OOO0O0OO0O000O0OO ,verify =False )#line:92
            OOO000O0OO0OO0O00 =OOO000O0OO0OO0O00 .json ()#line:93
            O0O0O0OOO000OOO00 =OOO000O0OO0OO0O00 ['result']#line:94
            OOO000O0OO0OO0O00 =OOO000O0OO0OO0O00 ['data']#line:95
            if O0O0O0OOO000OOO00 =='error':#line:96
                raise Exception (f'{OOO000O0OO0OO0O00}')#line:97
            OO000000O0O000OO0 =',\n'.join ([str (O00O0OOO0OOOOO00O )for O00O0OOO0OOOOO00O in OOO000O0OO0OO0O00 ])#line:98
            print (f'Dataset: {OO000000O0O000OO0} ')#line:99
            if OO0O0OO0O000OO000 =='e3':#line:100
                if len (OOO000O0OO0OO0O00 )<2 :#line:101
                    if O00OO0000OO000OO0 <=0 :#line:102
                        raise Exception ('超时且数据为空')#line:103
                    else :#line:104
                        time .sleep (O0O0OO00O0O0OOO0O )#line:105
                        O00OO0000OO000OO0 =O00OO0000OO000OO0 -O0O0OO00O0O0OOO0O #line:106
                else :#line:107
                    break #line:108
            else :#line:109
                if len (OOO000O0OO0OO0O00 )>1 :#line:110
                    if OO0O0OO0O000OO000 =='e1':#line:111
                        raise Exception ('有异常数据')#line:112
                    elif OO0O0OO0O000OO000 =='e2':#line:113
                        list_email (f'Info_{OO00O0O000OOO0OO0}',f'{OO00O0O000OOO0OO0}-Dataset Status',OOO000O0OO0OO0O00 ,to_list =tolist )#line:114
                else :#line:115
                    if OO0O0OO0O000OO000 not in ['info','e1']:#line:116
                        OO000000O0O000OO0 ='数据为空'#line:117
                        raise Exception (OO000000O0O000OO0 )#line:118
                break #line:119
    except Exception as O000OO000OOOOOOO0 :#line:120
        fun_email (f'{OO00O0O000OOO0OO0}-执行Dataset校验出错',OO000000O0O000OO0 ,to_list =tolist )#line:121
        raise SmartPipError (str (O000OO000OOOOOOO0 .args ))#line:122
def refresh_dash (OOOO0OOO0O00O0000 ,O0OO0O0O0O0OOO000 ):#line:125
    ""#line:128
    try :#line:129
        OO0OOOOOOOOO0O00O =requests .get (f'{_O0OO0OO00OO0OO00O}{O0OO0O0O0O0OOO000}',verify =False )#line:130
        OO0OOOOOOOOO0O00O =OO0OOOOOOOOO0O00O .json ()#line:131
        print (OO0OOOOOOOOO0O00O )#line:132
        O0OO00OO00OOO0OOO =OO0OOOOOOOOO0O00O ['status']#line:133
        if O0OO00OO00OOO0OOO !=200 :#line:134
            raise SmartPipError ('refresh_dash')#line:135
    except Exception as OOO0O0OO0O0O00OO0 :#line:136
        fun_email (f'{OOOO0OOO0O00O0000}-执行refresh出错',str (OOO0O0OO0O0O00OO0 .args ))#line:137
        raise SmartPipError (str (OOO0O0OO0O0O00OO0 .args ))#line:138
def refresh_quality (OOO00O0OO0OO0000O ,OOOOOOO0000000O0O ,hours =1 ):#line:140
    ""#line:143
    try :#line:144
        OO00OOO0OO0OO000O =requests .get (f'{_O0OO0OO00OO0OO00O}{OOOOOOO0000000O0O}&hours={hours}',verify =False )#line:145
        OO00OOO0OO0OO000O =OO00OOO0OO0OO000O .json ()#line:146
        print (OO00OOO0OO0OO000O )#line:147
        OOO0OOO0O000OOOOO =OO00OOO0OO0OO000O ['status']#line:148
        if OOO0OOO0O000OOOOO !=200 :#line:149
            raise SmartPipError ('refresh_quality')#line:150
    except Exception as O0O0OO0O000000O00 :#line:151
        fun_email (f'{OOO00O0OO0OO0000O}-执行refresh_quality出错',str (O0O0OO0O000000O00 .args ))#line:152
        raise SmartPipError (str (O0O0OO0O000000O00 .args ))#line:153
def run_bash (O000OOOOOO0OOO0OO ):#line:157
    OOO0OO0O000O0000O =''#line:158
    O0OO00O0OO0OO0OOO =subprocess .Popen (O000OOOOOO0OOO0OO ,stdout =subprocess .PIPE ,stderr =subprocess .STDOUT ,shell =True ,cwd =ETL_FILE_PATH )#line:159
    print ('PID:',O0OO00O0OO0OO0OOO .pid )#line:160
    for OOO0OOOOO00O0OOO0 in iter (O0OO00O0OO0OO0OOO .stdout .readline ,b''):#line:161
        if O0OO00O0OO0OO0OOO .poll ()and OOO0OOOOO00O0OOO0 ==b'':#line:162
            break #line:163
        OOO0OOOOO00O0OOO0 =OOO0OOOOO00O0OOO0 .decode (encoding ='utf8')#line:164
        print (OOO0OOOOO00O0OOO0 .rstrip ())#line:165
        OOO0OO0O000O0000O =OOO0OO0O000O0000O +OOO0OOOOO00O0OOO0 #line:166
    O0OO00O0OO0OO0OOO .stdout .close ()#line:167
    O00000OOO00O00000 =O0OO00O0OO0OO0OOO .wait ()#line:168
    print ('result code: ',O00000OOO00O00000 )#line:169
    return OOO0OO0O000O0000O ,O00000OOO00O00000 #line:170
def run_python (O0O0OOOO0OO0OO00O ,OOOO0O0O00O0OO00O ,dev =''):#line:173
    O00OO0OOO0OO00OO0 =O0O0OOOO0OO0OO00O .split ('/')#line:174
    _O0OOOOO0O0O0O000O ,OO0O000O000OO0OOO =run_bash ('python %s %s'%(O0O0OOOO0OO0OO00O ,OOOO0O0O00O0OO00O ))#line:175
    if OO0O000O000OO0OOO !=0 :#line:176
        fun_email (f'{O00OO0OOO0OO00OO0[-2]}/{O00OO0OOO0OO00OO0[-1]}出错','python error')#line:177
        raise Exception ('error')#line:178
def run_dataxx (O0O00OOO0000O0000 ,OOO00O00OOOOO0000 ,dev =''):#line:182
    O00O00OO000000O0O =O0O00OOO0000O0000 .split ('/')#line:183
    if OOO00O00OOOOO0000 :#line:184
        OOO0OO0OOO0O00OO0 =[f'-D{O0OO0O0O0OO0O0OOO}:{O000OO00OO000OOO0}'for O0OO0O0O0OO0O0OOO ,O000OO00OO000OOO0 in OOO00O00OOOOO0000 .items ()]#line:185
        O0OOO0O0O00000O00 =' '.join (OOO0OO0OOO0O00OO0 )#line:186
        O0OOO000OOO00O000 =[f'-p"{O0OOO0O0O00000O00}"',O0O00OOO0000O0000 ]#line:187
    else :#line:188
        O0OOO000OOO00O000 =[O0O00OOO0000O0000 ]#line:189
    O0O0O0O0OO0OO00O0 =datax_cmdStr (O0OOO000OOO00O000 )#line:190
    _OO0OOOOO0OO0OO000 ,OOOOO0OOO00O0OOOO =run_bash (O0O0O0O0OO0OO00O0 )#line:191
    if OOOOO0OOO00O0OOOO !=0 :#line:192
        fun_email (f'{O00O00OO000000O0O[-2]}/{O00O00OO000000O0O[-1]}出错','datax error')#line:193
        raise Exception ('error')#line:194
def run_datax (O00OO00OOO0OOO0OO ,O0O0O0OO0O0O0OO0O ,OOOO00OO0OO000000 ,OO00O0OO00OOO000O ,dev =''):#line:197
    with open (O00OO00OOO0OOO0OO ,'r',encoding ='utf8')as OO0000O0O0000OOO0 :#line:198
        OOOO0O0O00O0O000O =readSqlstr (OO0000O0O0000OOO0 .read ().strip (),para_dict =OO00O0OO00OOO000O )#line:199
    OOOO0O0O00O0O000O =OOOO0O0O00O0O000O .split ('##')#line:200
    O00O0O00OO0O00OOO ={}#line:201
    for OOO0OOOO0O0O000O0 in OOOO0O0O00O0O000O :#line:202
        O0O0OOO000OO0O00O =OOO0OOOO0O0O000O0 .find ('=')#line:203
        if O0O0OOO000OO0O00O >0 :#line:204
            O00O0O00OO0O00OOO [OOO0OOOO0O0O000O0 [:O0O0OOO000OO0O00O ].strip ()]=OOO0OOOO0O0O000O0 [O0O0OOO000OO0O00O +1 :].replace ('\n',' ').strip ()#line:205
    O00O0OOOOOO0OO0OO =O00O0O00OO0O00OOO .keys ()#line:206
    OOO0OO00O00OO0O0O =O00O0O00OO0O00OOO .pop ('template')if 'template'in O00O0OOOOOO0OO0OO else 'default'#line:207
    OO0OOOOOOOOO0OOO0 =O00O0O00OO0O00OOO .get ('targetColumn')#line:208
    O0O0OOOOO000OOOOO =None #line:209
    if OOO0OO00O00OO0O0O .endswith ('hdfs'):#line:210
        O0O0OOOOO000OOOOO =O00O0O00OO0O00OOO .pop ('hiveSql')if 'hiveSql'in O00O0OOOOOO0OO0OO else None #line:212
        if not O0O0OOOOO000OOOOO :#line:213
            O0O0OOOOO000OOOOO =O00O0O00OO0O00OOO .pop ('postSql')if 'postSql'in O00O0OOOOOO0OO0OO else None #line:214
        if OO0OOOOOOOOO0OOO0 :#line:216
            OO0OOOOOOOOO0OOO0 =OO0OOOOOOOOO0OOO0 .split (',')#line:217
            OOOO0000OOOO000O0 =[]#line:218
            for OOO0OOOO0O0O000O0 in OO0OOOOOOOOO0OOO0 :#line:219
                if ':'in OOO0OOOO0O0O000O0 :#line:220
                    OOO0OOOO0O0O000O0 =OOO0OOOO0O0O000O0 .split (':')#line:221
                    OOOO0000OOOO000O0 .append ({"name":OOO0OOOO0O0O000O0 [0 ].strip (),"type":OOO0OOOO0O0O000O0 [1 ].strip ()})#line:222
                else :#line:223
                    OOOO0000OOOO000O0 .append ({"name":OOO0OOOO0O0O000O0 .strip (),"type":"STRING"})#line:224
            O00O0O00OO0O00OOO ['targetColumn']=json .dumps (OOOO0000OOOO000O0 )#line:225
    else :#line:226
        if OO0OOOOOOOOO0OOO0 :#line:227
            OO0OOOOOOOOO0OOO0 =[OOO00OOO00000000O .strip ()for OOO00OOO00000000O in OO0OOOOOOOOO0OOO0 .split (',')]#line:228
            O00O0O00OO0O00OOO ['targetColumn']=json .dumps (OO0OOOOOOOOO0OOO0 )#line:229
        else :#line:230
            O00O0O00OO0O00OOO ['targetColumn']='["*"]'#line:231
        if OOO0OO00O00OO0O0O .endswith ('starrocks'):#line:233
            if '.'in O00O0O00OO0O00OOO ['targetTable']:#line:234
                O00O0O00OO0O00OOO ['targetDB'],O00O0O00OO0O00OOO ['targetTable']=O00O0O00OO0O00OOO ['targetTable'].split ('.')#line:235
            else :#line:236
                O00O0O00OO0O00OOO ['targetDB']='Test'#line:237
    if 'preSql'in O00O0OOOOOO0OO0OO :#line:239
        O00O0O00OO0O00OOO ['preSql']=json .dumps (O00O0O00OO0O00OOO ['preSql'].strip ().split (';'))#line:240
    else :#line:241
        O00O0O00OO0O00OOO ['preSql']=''#line:242
    if 'postSql'in O00O0OOOOOO0OO0OO :#line:243
        O00O0O00OO0O00OOO ['postSql']=json .dumps (O00O0O00OO0O00OOO ['postSql'].strip ().split (';'))#line:244
    else :#line:245
        O00O0O00OO0O00OOO ['postSql']=''#line:246
    OO00O0OO0O0000OO0 =O00OO00OOO0OOO0OO .split ('/')#line:247
    OO00000O00O000O0O =OO00O0OO0O0000OO0 [-1 ].split ('.')[0 ]#line:248
    with open (os .path .join (OOOO00OO0OO000000 ,'datax','templates',OOO0OO00O00OO0O0O ),'r')as OO0000O0O0000OOO0 :#line:249
        OOOOOOOO0000OO000 =OO0000O0O0000OOO0 .read ()#line:250
    O00OO00OOO0OOO0OO =os .path .join (OOOO00OO0OO000000 ,'datax',OO00000O00O000O0O +'.json')#line:251
    with open (O00OO00OOO0OOO0OO ,'w',encoding ='utf8')as OO0000O0O0000OOO0 :#line:252
        OO0000O0O0000OOO0 .write (readSqlstr (OOOOOOOO0000OO000 ,O00O0O00OO0O00OOO ))#line:253
    OOOOO0OOOOO0OO0OO =datax_cmdStr ([O00OO00OOO0OOO0OO ])#line:254
    _OOO0O00OOOO000000 ,OO0000000OO0OO0O0 =run_bash (OOOOO0OOOOO0OO0OO )#line:255
    if OO0000000OO0OO0O0 !=0 :#line:256
        fun_email (f'{OO00O0OO0O0000OO0[-2]}/{OO00O0OO0O0000OO0[-1]}出错','datax error')#line:257
        raise Exception ('error')#line:258
    if O0O0OOOOO000OOOOO :#line:259
        print (_OO000OO0O0O000OO0 (O0O0OOOOO000OOOOO .split (';'),O0O0O0OO0O0O0OO0O ,db_connect ='hive',dev =dev ))#line:260
def readSqlFile (OOOOO0O000O00O0OO ,para_dict =None ):#line:264
    if OOOOO0O000O00O0OO .find ('.sql')<0 :#line:265
        return 'file type error'#line:266
    with open (OOOOO0O000O00O0OO ,'r',encoding ='utf-8')as O000OOOO0OO0OOOOO :#line:267
        O000OO00OO0OOOOOO =O000OOOO0OO0OOOOO .read ()#line:268
    O000O00OO00O0O0O0 =readSqlstr (O000OO00OO0OOOOOO ,para_dict )#line:269
    return O000O00OO00O0O0O0 #line:270
def readSqoopFile (O0OO00OOO0OOOOO0O ,para_dict =None ):#line:273
    if not O0OO00OOO0OOOOO0O .endswith ('.sql'):#line:274
        return 'file type error'#line:275
    with open (O0OO00OOO0OOOOO0O ,'r',encoding ='utf8')as OO0OO0O00OOOOOOO0 :#line:276
        OO0O0OOO000OOO00O =OO0OO0O00OOOOOOO0 .read ().strip ()#line:277
    O000OO0O0O0OOO00O =re .match (r"/\*(.*?)\*/(.+)",OO0O0OOO000OOO00O ,re .M |re .S )#line:278
    OOOO000O0000O00O0 =readSqlstr (O000OO0O0O0OOO00O .group (1 ).strip (),para_dict )#line:279
    O0OO0O00O0000O00O =O000OO0O0O0OOO00O .group (2 ).strip ()#line:280
    return OOOO000O0000O00O0 ,O0OO0O00O0000O00O #line:281
def readSqlstr (OOO0OO000O0O0O00O ,para_dict =None ):#line:284
    OOOO00O000OO0OOOO =re .sub (r"(\/\*(.|\n)*?\*\/)|--.*",'',OOO0OO000O0O0O00O .strip ())#line:285
    if para_dict :#line:286
        for OO000OO0OOO0OO0OO ,O00O0O00OO00OO000 in para_dict .items ():#line:287
            OOOO00O000OO0OOOO =OOOO00O000OO0OOOO .replace ('$'+OO000OO0OOO0OO0OO ,str (O00O0O00OO00OO000 ))#line:288
    return OOOO00O000OO0OOOO #line:289
def run_sql_file (O00OO0OOOOOOOOOOO ,O00000000OOOOOO00 ,db_connect ='starrocks',para_dict =None ,dev =''):#line:292
    O0000O00O0OO0OO00 =O00OO0OOOOOOOOOOO .split ('/')#line:293
    try :#line:294
        OO0O000OO0OO00OO0 =readSqlFile (O00OO0OOOOOOOOOOO ,para_dict ).split (';')#line:295
        OO0O0000O0OOO000O =O00000000OOOOOO00 .get (db_connect )#line:296
        if dev :#line:297
            if f'{db_connect}{dev}'in O00000000OOOOOO00 .keys ():#line:298
                OO0O0000O0OOO000O =O00000000OOOOOO00 .get (f'{db_connect}{dev}')#line:299
        OOOOOOO0O000OOO0O =connect_db_execute ().execute_sql_list (OO0O000OO0OO00OO0 ,db_connect ,connect_dict =OO0O0000O0OOO000O )#line:300
        return OOOOOOO0O000OOO0O #line:301
    except Exception as OO0000O0OOOOO0OOO :#line:302
        fun_email ('{}/{}执行出错'.format (O0000O00O0OO0OO00 [-2 ],O0000O00O0OO0OO00 [-1 ]),str (OO0000O0OOOOO0OOO .args ))#line:303
        print (OO0000O0OOOOO0OOO .args )#line:304
        raise SmartPipError ('Run SQL Error')#line:305
def _OO000OO0O0O000OO0 (O00O0000O00000O00 ,OOOOO0OOO0O0O0O0O ,db_connect ='starrocks',para_dict =None ,dev =''):#line:308
    try :#line:309
        if isinstance (O00O0000O00000O00 ,str ):#line:310
            O00O0000O00000O00 =readSqlstr (O00O0000O00000O00 ,para_dict ).split (';')#line:311
        OO0O0O00000OO0000 =OOOOO0OOO0O0O0O0O .get (db_connect )#line:312
        if dev :#line:313
            if f'{db_connect}{dev}'in OOOOO0OOO0O0O0O0O .keys ():#line:314
                OO0O0O00000OO0000 =OOOOO0OOO0O0O0O0O .get (f'{db_connect}{dev}')#line:315
        O0OOO00O000000OO0 =connect_db_execute ().execute_sql_list (O00O0000O00000O00 ,db_connect ,connect_dict =OO0O0O00000OO0000 )#line:316
        return O0OOO00O000000OO0 #line:317
    except Exception as O0OOO00000OOO00O0 :#line:318
        fun_email ('SQL执行出错',f'{O00O0000O00000O00}{O0OOO00000OOO00O0.args}')#line:319
        print (O0OOO00000OOO00O0 .args )#line:320
        raise SmartPipError ('Run SQL Error')#line:321
def run_kettle (O00000O00O0OOOO0O ,para_str ='',dev =False ):#line:325
    ""#line:332
    O0OOOO00000OOO0OO =O00000O00O0OOOO0O .split ('/')#line:333
    print ('kettle job start')#line:334
    if '.ktr'in O00000O00O0OOOO0O :#line:336
        OOO0000OO000OOO0O =f'{KETTLE_HOME}/pan.sh -level=Basic -file={O00000O00O0OOOO0O}{para_str}'#line:337
    else :#line:338
        OOO0000OO000OOO0O =f'{KETTLE_HOME}/kitchen.sh -level=Basic -file={O00000O00O0OOOO0O}{para_str}'#line:339
    print (OOO0000OO000OOO0O )#line:340
    OOOOOO00O0OO00OOO ,O00OO000OO0OOOOO0 =run_bash (OOO0000OO000OOO0O )#line:344
    if O00OO000OO0OOOOO0 ==0 :#line:345
        print ('{} 完成数据抽取'.format (str (O00000O00O0OOOO0O )))#line:346
    else :#line:347
        print ('{} 执行错误'.format (O00000O00O0OOOO0O ))#line:348
        fun_email ('{}/{}出错'.format (O0OOOO00000OOO0OO [-2 ],O0OOOO00000OOO0OO [-1 ]),str (OOOOOO00O0OO00OOO ))#line:349
        raise SmartPipError ('Run Kettle Error')#line:350
def hdfsStarrocks (O0O0OOO0OO0OO0OOO ,OO0O0O0OO00O0O00O ,para_dict =None ):#line:354
    ""#line:358
    OO00OOOOOOOO0OO0O =O0O0OOO0OO0OO0OOO .split ('/')#line:359
    print ('starrocks load job start')#line:360
    OOOO0OO0OO0OO00OO ,O0OOOOO00O000O0OO =readSqoopFile (O0O0OOO0OO0OO0OOO ,para_dict =para_dict )#line:361
    OOOO0OO0OO0OO00OO =OOOO0OO0OO0OO00OO .split ('\n')#line:362
    OOOO0O00OO000000O ={}#line:363
    OOOO0O00OO000000O ['LABEL']=f'{OO00OOOOOOOO0OO0O[-2]}{OO00OOOOOOOO0OO0O[-1][:-4]}{int(time.time())}'#line:364
    OOOO0O00OO000000O ['HDFS']=HIVE_HOME #line:365
    for O000O0OOOOOO0O0O0 in OOOO0OO0OO0OO00OO :#line:366
        O0O000OO0OOO0O0O0 =O000O0OOOOOO0O0O0 .find ('=')#line:367
        if O0O000OO0OOO0O0O0 >0 :#line:368
            OOOO0O00OO000000O [O000O0OOOOOO0O0O0 [:O0O000OO0OOO0O0O0 ].strip ()]=O000O0OOOOOO0O0O0 [O0O000OO0OOO0O0O0 +1 :].strip ()#line:369
    OO00O00OO0OO0000O =OOOO0O00OO000000O .get ('sleepTime')#line:371
    if OO00O00OO0OO0000O :#line:372
        OO00O00OO0OO0000O =int (OO00O00OO0OO0000O )#line:373
        if OO00O00OO0OO0000O <30 :#line:374
            OO00O00OO0OO0000O =30 #line:375
    else :#line:376
        OO00O00OO0OO0000O =30 #line:377
    OOO00OOOOO00OOO00 =OOOO0O00OO000000O .get ('maxTime')#line:379
    if OOO00OOOOO00OOO00 :#line:380
        OOO00OOOOO00OOO00 =int (OOO00OOOOO00OOO00 )#line:381
        if OOO00OOOOO00OOO00 >3600 :#line:382
            OOO00OOOOO00OOO00 =3600 #line:383
    else :#line:384
        OOO00OOOOO00OOO00 =600 #line:385
    _OO000OO0O0O000OO0 (O0OOOOO00O000O0OO ,OO0O0O0OO00O0O00O ,db_connect ='starrocks',para_dict =OOOO0O00OO000000O )#line:387
    time .sleep (OO00O00OO0OO0000O )#line:388
    OO0O00000O00O0000 =f'''show load from {OOOO0O00OO000000O.get('targetDB')} where label = '{OOOO0O00OO000000O['LABEL']}' order by CreateTime desc limit 1 '''#line:389
    O0OOOO00OOOO000O0 ='start to check label'#line:390
    try :#line:391
        while True :#line:392
            O0OOOO00OOOO000O0 =_OO000OO0O0O000OO0 ([OO0O00000O00O0000 ],OO0O0O0OO00O0O00O ,db_connect ='starrocks')#line:393
            print (O0OOOO00OOOO000O0 )#line:394
            OOOOO0OOOOOO0OOOO =O0OOOO00OOOO000O0 [1 ][2 ]#line:395
            if OOOOO0OOOOOO0OOOO =='CANCELLED':#line:396
                raise Exception (f'Starrocks:{OOOOO0OOOOOO0OOOO}')#line:397
            elif OOOOO0OOOOOO0OOOO =='FINISHED':#line:398
                print ('Load completed')#line:399
                break #line:400
            if OOO00OOOOO00OOO00 <=0 :#line:401
                raise Exception ('超时未完成')#line:402
            else :#line:403
                time .sleep (OO00O00OO0OO0000O )#line:404
                OOO00OOOOO00OOO00 =OOO00OOOOO00OOO00 -OO00O00OO0OO0000O #line:405
    except Exception as O00O0000OO0O0O0O0 :#line:406
        print ('{} 执行错误'.format (O0O0OOO0OO0OO0OOO ))#line:407
        fun_email ('{}/{}执行出错'.format (OO00OOOOOOOO0OO0O [-2 ],OO00OOOOOOOO0OO0O [-1 ]),str (O0OOOO00OOOO000O0 ))#line:408
        raise SmartPipError (str (O00O0000OO0O0O0O0 .args ))#line:409
def kafkaStarrocks (OO00O0OO0OO0OO0OO ,O000OO0OOOOO000OO ,OOO00OOOOO0OO0OOO ,OO0000O0000OOO0OO ,O00OO0OOO0OO0O0O0 ,dev =''):#line:412
    with open (OO00O0OO0OO0OO0OO ,'r',encoding ='utf8')as OOOOO0OO0000OOOO0 :#line:413
        O00O00O00000O0OO0 =readSqlstr (OOOOO0OO0000OOOO0 .read ().strip (),para_dict =O00OO0OOO0OO0O0O0 )#line:414
    O00O00O00000O0OO0 =O00O00O00000O0OO0 .split ('##')#line:415
    O00OO0O00OO0OOO0O ={}#line:416
    for O0O00OOOO0OO000O0 in O00O00O00000O0OO0 :#line:417
        O0000OO0OOO0OOO00 =O0O00OOOO0OO000O0 .find ('=')#line:418
        if O0000OO0OOO0OOO00 >0 :#line:419
            O00O00OO0OO00O0O0 =O0O00OOOO0OO000O0 [O0000OO0OOO0OOO00 +1 :].replace ('\n',' ').strip ()#line:420
            if O00O00OO0OO00O0O0 :#line:421
                O00OO0O00OO0OOO0O [O0O00OOOO0OO000O0 [:O0000OO0OOO0OOO00 ].strip ()]=O00O00OO0OO00O0O0 #line:422
    O000O0000O00O0000 =O00OO0O00OO0OOO0O .pop ('topic')#line:423
    OO000O0O0000OOO00 =O00OO0O00OO0OOO0O .pop ('table')#line:424
    O0O0OO000OOOOOO00 =O00OO0O00OO0OOO0O .keys ()#line:425
    if 'skipError'in O0O0OO000OOOOOO00 :#line:426
        skipError =O00OO0O00OO0OOO0O .pop ('skipError')#line:427
    else :#line:428
        skipError =None #line:429
    if 'kafkaConn'in O0O0OO000OOOOOO00 :#line:430
        O0O0O00OOOO0000O0 =O00OO0O00OO0OOO0O .pop ('kafkaConn')#line:431
    else :#line:432
        O0O0O00OOOO0000O0 ='default'#line:433
    if 'offsetDict'in O0O0OO000OOOOOO00 :#line:434
        O00O00OO000OOOO00 =json .loads (O00OO0O00OO0OOO0O .pop ('offsetDict'))#line:435
    else :#line:436
        O00O00OO000OOOO00 =None #line:437
    if 'json_root'in O0O0OO000OOOOOO00 :#line:438
        OO00O0O00O00OO000 =O00OO0O00OO0OOO0O .pop ('json_root')#line:439
    else :#line:440
        OO00O0O00O00OO000 =None #line:441
    if 'jsonpaths'in O0O0OO000OOOOOO00 :#line:442
        O0000000O0O0OOOOO =O00OO0O00OO0OOO0O .get ('jsonpaths')#line:443
        if not O0000000O0O0OOOOO .startswith ('['):#line:444
            O0000000O0O0OOOOO =O0000000O0O0OOOOO .split (',')#line:445
            O0000000O0O0OOOOO =json .dumps (['$.'+OOOOO00O0OO00OO0O .strip ()for OOOOO00O0OO00OO0O in O0000000O0O0OOOOO ])#line:446
            O00OO0O00OO0OOO0O ['jsonpaths']=O0000000O0O0OOOOO #line:447
    OOO0O000O00O0OOO0 =_OO00O0OOO000O00O0 (O000O0000O00O0000 ,O000OO0OOOOO000OO [O0O0O00OOOO0000O0 ],OO0000O0000OOO0OO ,O00O00OO000OOOO00 )#line:448
    def O00O0O00OO00OO0O0 (OOOO0OOOOO00O0OOO ):#line:450
        OO0OOO0O0OOO0OO0O =b''#line:451
        O0OO0OOOO0O0OOOOO =None #line:452
        if 'format'in O0O0OO000OOOOOO00 :#line:453
            for O0OO0OOOO0O0OOOOO in OOOO0OOOOO00O0OOO :#line:454
                O0000OO00OO000OOO =O0OO0OOOO0O0OOOOO .value #line:455
                if OO00O0O00O00OO000 :#line:456
                    O0000OO00OO000OOO =json .loads (O0000OO00OO000OOO .decode ('utf8'))#line:457
                    O0000OO00OO000OOO =json .dumps (O0000OO00OO000OOO [OO00O0O00O00OO000 ]).encode ('utf8')#line:458
                if O0000OO00OO000OOO .startswith (b'['):#line:459
                    OO0OOO0O0OOO0OO0O =OO0OOO0O0OOO0OO0O +b','+O0000OO00OO000OOO [1 :-1 ]#line:460
                else :#line:461
                    OO0OOO0O0OOO0OO0O =OO0OOO0O0OOO0OO0O +b','+O0000OO00OO000OOO #line:462
                if len (OO0OOO0O0OOO0OO0O )>94857600 :#line:463
                    streamStarrocks (OO000O0O0000OOO00 ,OOO00OOOOO0OO0OOO ,O00OO0O00OO0OOO0O ,OO0OOO0O0OOO0OO0O ,skipError )#line:464
                    OOO0O000O00O0OOO0 .write_offset (O0OO0OOOO0O0OOOOO .partition ,O0OO0OOOO0O0OOOOO .offset +1 )#line:465
                    OO0OOO0O0OOO0OO0O =b''#line:466
                if O0OO0OOOO0O0OOOOO .offset ==OOO0O000O00O0OOO0 .end_offset -1 :#line:467
                    break #line:468
        else :#line:469
            for O0OO0OOOO0O0OOOOO in OOOO0OOOOO00O0OOO :#line:470
                O0000OO00OO000OOO =O0OO0OOOO0O0OOOOO .value #line:471
                if OO00O0O00O00OO000 :#line:472
                    O0000OO00OO000OOO =json .loads (O0000OO00OO000OOO .decode ('utf8'))#line:473
                    O0000OO00OO000OOO =json .dumps (O0000OO00OO000OOO [OO00O0O00O00OO000 ]).encode ('utf8')#line:474
                OO0OOO0O0OOO0OO0O =OO0OOO0O0OOO0OO0O +b'\n'+O0000OO00OO000OOO #line:475
                if len (OO0OOO0O0OOO0OO0O )>94857600 :#line:476
                    streamStarrocks (OO000O0O0000OOO00 ,OOO00OOOOO0OO0OOO ,O00OO0O00OO0OOO0O ,OO0OOO0O0OOO0OO0O ,skipError )#line:477
                    OOO0O000O00O0OOO0 .write_offset (O0OO0OOOO0O0OOOOO .partition ,O0OO0OOOO0O0OOOOO .offset +1 )#line:478
                    OO0OOO0O0OOO0OO0O =b''#line:479
                if O0OO0OOOO0O0OOOOO .offset ==OOO0O000O00O0OOO0 .end_offset -1 :#line:480
                    break #line:481
        print (OO0OOO0O0OOO0OO0O [1 :1000 ])#line:482
        if OO0OOO0O0OOO0OO0O :#line:483
            streamStarrocks (OO000O0O0000OOO00 ,OOO00OOOOO0OO0OOO ,O00OO0O00OO0OOO0O ,OO0OOO0O0OOO0OO0O ,skipError )#line:484
        return O0OO0OOOO0O0OOOOO #line:485
    OOO0O000O00O0OOO0 .consumer_topic (O00O0O00OO00OO0O0 )#line:487
def apiStarrocks (O0O0OO0O0OO0000O0 ,O00O0OOO0O0OOO00O ,O0000O00O0000O0OO ,OOO0OO00O0OOOOO00 ):#line:490
    with open (O0O0OO0O0OO0000O0 ,'r',encoding ='utf8')as O0O0OO0O000OO0000 :#line:491
        OO0OO0O0O0000OOOO =readSqlstr (O0O0OO0O000OO0000 .read ().strip (),para_dict =OOO0OO00O0OOOOO00 )#line:492
    OO0OO0O0O0000OOOO =OO0OO0O0O0000OOOO .split ('##')#line:493
    O00O0OO000OO00OOO ={}#line:494
    for O0000000O00OO000O in OO0OO0O0O0000OOOO :#line:495
        O00OOO0OO00000O0O =O0000000O00OO000O .find ('=')#line:496
        if O00OOO0OO00000O0O >0 :#line:497
            O00000O00O0O0O00O =O0000000O00OO000O [O00OOO0OO00000O0O +1 :].replace ('\n',' ').strip ()#line:498
            if O00000O00O0O0O00O :#line:499
                O00O0OO000OO00OOO [O0000000O00OO000O [:O00OOO0OO00000O0O ].strip ()]=O00000O00O0O0O00O #line:500
    OOO00O0000O00OO00 =O00O0OO000OO00OOO .pop ('table')#line:501
    OOOOO0O0O00000000 =O00O0OO000OO00OOO .keys ()#line:502
    if 'param'in OOOOO0O0O00000000 :#line:503
        OOO00O0O00OO0OO0O =O00O0OO000OO00OOO .pop ('param')#line:504
    else :#line:505
        OOO00O0O00OO0OO0O =None #line:506
    if 'apiConn'in OOOOO0O0O00000000 :#line:507
        O0O000000O000OO00 =O00O0OO000OO00OOO .pop ('apiConn')#line:508
    else :#line:509
        O0O000000O000OO00 ='default'#line:510
    if 'skipError'in OOOOO0O0O00000000 :#line:511
        skipError =O00O0OO000OO00OOO .pop ('skipError')#line:512
    else :#line:513
        skipError =None #line:514
    if 'jsonpaths'in OOOOO0O0O00000000 :#line:515
        O0OOO0O000O000000 =O00O0OO000OO00OOO .get ('jsonpaths')#line:516
        if not O0OOO0O000O000000 .startswith ('['):#line:517
            O0OOO0O000O000000 =O0OOO0O000O000000 .split (',')#line:518
            O0OOO0O000O000000 =json .dumps (['$.'+O0OO0O00OO0000O0O .strip ()for O0OO0O00OO0000O0O in O0OOO0O000O000000 ])#line:519
            O00O0OO000OO00OOO ['jsonpaths']=O0OOO0O000O000000 #line:520
    OOOO00O00000OOOOO =O00O0OOO0O0OOO00O [O0O000000O000OO00 ](OOO00O0O00OO0OO0O )#line:521
    if OOOO00O00000OOOOO :#line:522
        streamStarrocks (OOO00O0000O00OO00 ,O0000O00O0000O0OO ,O00O0OO000OO00OOO ,OOOO00O00000OOOOO ,skipError )#line:523
    else :#line:524
        print ('无数据')#line:525
def streamStarrocks (OO0OOO0OOOO00OO0O ,O00O0OO0O0O000OO0 ,O0O0000OOOOOOOOOO ,OO000OO0000O00000 ,skipError =False ):#line:528
    ""#line:531
    import base64 ,uuid #line:532
    O0O0O0O0OOOOOOOO0 ,OO0OOO0OOOO00OO0O =OO0OOO0OOOO00OO0O .split ('.')#line:533
    O00O0OOOOO00O0O00 =str (base64 .b64encode (f'{O00O0OO0O0O000OO0["user"]}:{O00O0OO0O0O000OO0["password"]}'.encode ('utf-8')),'utf-8')#line:534
    OO000OO0000O00000 =OO000OO0000O00000 .strip ()#line:535
    if OO000OO0000O00000 .startswith (b','):#line:536
        O0O0000OOOOOOOOOO ['strip_outer_array']='true'#line:537
        OO000OO0000O00000 =b'['+OO000OO0000O00000 [1 :]+b']'#line:538
    O0OO0OOOO00000O00 ={'Content-Type':'application/json','Authorization':f'Basic {O00O0OOOOO00O0O00}','label':f'{OO0OOO0OOOO00OO0O}{uuid.uuid4()}',**O0O0000OOOOOOOOOO }#line:544
    O0O0O0OOOO00O0O0O =f"{O00O0OO0O0O000OO0['url']}/api/{O0O0O0O0OOOOOOOO0}/{OO0OOO0OOOO00OO0O}/_stream_load"#line:545
    print ('start loading to starrocks....')#line:546
    O0O0O00OO00000OOO =requests .put (O0O0O0OOOO00O0O0O ,headers =O0OO0OOOO00000O00 ,data =OO000OO0000O00000 ).json ()#line:547
    print (O0O0O00OO00000OOO )#line:548
    if O0O0O00OO00000OOO ['Status']=='Fail':#line:549
        if skipError :#line:550
            print (f'Starrocks Load Error, Skip this offset')#line:551
        else :#line:552
            raise Exception ('Starrocks Load Error')#line:553
def routineStarrocks (OOOO0O0O0OO0000O0 ,OO0O0OO00O00O0OOO ,flag =''):#line:556
    O0000OOO0O000O000 =_OO000OO0O0O000OO0 ([f'SHOW ROUTINE LOAD FOR {OO0O0OO00O00O0OOO}'],OOOO0O0O0OO0000O0 ,db_connect ='starrocks')#line:557
    O0000OOO0O000O000 =dict (zip (O0000OOO0O000O000 [0 ],O0000OOO0O000O000 [1 ]))#line:558
    print ('状态:',O0000OOO0O000O000 ['State'])#line:559
    print ('统计:',O0000OOO0O000O000 ['Statistic'])#line:560
    print ('进度:',O0000OOO0O000O000 ['Progress'])#line:561
    if O0000OOO0O000O000 ['State']!='RUNNING':#line:562
        print ('ERROR: ',O0000OOO0O000O000 ['ReasonOfStateChanged'])#line:563
        if not flag :#line:564
            raise Exception ('Starrocks Routin Load')#line:565
from airflow .utils .session import provide_session #line:571
@provide_session #line:572
def point_test (OOO000OOOO0O000OO ,sleeptime ='',maxtime ='',session =None ):#line:573
    ""#line:580
    if sleeptime :#line:581
        sleeptime =int (sleeptime )#line:582
        sleeptime =sleeptime if sleeptime >60 else 60 #line:583
    if maxtime :#line:584
        maxtime =int (maxtime )#line:585
        maxtime =maxtime if maxtime <60 *60 *2 else 60 *60 *2 #line:586
    else :#line:587
        maxtime =0 #line:588
    try :#line:589
        OOO000OO0O0O000OO =f"select start_date,state from dag_run where dag_id ='{OOO000OOOO0O000OO}' ORDER BY id desc LIMIT 1"#line:590
        while True :#line:591
            O0O00OO0OOOO0OOOO =session .execute (OOO000OO0O0O000OO ).fetchall ()#line:592
            if O0O00OO0OOOO0OOOO [0 ][1 ]!='success':#line:593
                if maxtime >0 and O0O00OO0OOOO0OOOO [0 ][1 ]!='failed':#line:594
                    print ('waiting...'+O0O00OO0OOOO0OOOO [0 ][1 ])#line:595
                    time .sleep (sleeptime )#line:596
                    maxtime =maxtime -sleeptime #line:597
                else :#line:598
                    O000OOO00OOO0O000 =O0O00OO0OOOO0OOOO [0 ][0 ].strftime ("%Y-%m-%d %H:%M:%S")#line:599
                    OO0000OO00OOOO0OO ='所依赖的dag:'+OOO000OOOO0O000OO +',状态为'+O0O00OO0OOOO0OOOO [0 ][1 ]+'.其最新的执行时间为'+O000OOO00OOO0O000 #line:600
                    fun_email (OO0000OO00OOOO0OO ,'前置DAG任务未成功')#line:601
                    print (OO0000OO00OOOO0OO )#line:602
                    raise SmartPipError ('Run DAG validate Error')#line:603
            else :#line:604
                print ('success...')#line:605
                break #line:606
    except Exception as O0OO000O0OO0O0O00 :#line:607
        print (O0OO000O0OO0O0O00 .args )#line:608
        raise SmartPipError ('DAG validate Error')#line:609
class connect_db_execute ():#line:614
    def __init__ (O0OO0OO000O000OO0 ,dev =False ,db =''):#line:615
        O0OO0OO000O000OO0 .dev =dev #line:616
    def insert_contents (OOO000OOO00000OO0 ,OO0O00O0O00000O0O ,OO000O0OOO0O0O0O0 ,per_in =1000 ,connect_dict =None ):#line:618
        OO0O0O000OOOO0OOO =time .time ()#line:619
        logging .info ('starting to execute insert contents...')#line:620
        if isinstance (connect_dict ,dict ):#line:621
            O0OO0O0000OO000OO =connect_dict ['dbtype']#line:622
        else :#line:623
            if connect_dict =='':#line:624
                O0OO0O0000OO000OO ='oracle'#line:625
            else :#line:626
                O0OO0O0000OO000OO =connect_dict #line:627
            connect_dict =None #line:628
        O0OO00O0O0000000O =getattr (OOO000OOO00000OO0 ,'insert_contents_'+O0OO0O0000OO000OO )#line:629
        OO0O0OOO0OOO00O0O =O0OO00O0O0000000O (OO0O00O0O00000O0O ,OO000O0OOO0O0O0O0 ,per_in ,connect_dict )#line:630
        logging .info ('execute insert contents time : {}ms'.format (time .time ()-OO0O0O000OOOO0OOO ))#line:631
        return OO0O0OOO0OOO00O0O #line:632
    def impala (OO0OOO0OO00O00OO0 ,OOOO0OO0OOOOO0O00 ,connect_dict =None ):#line:634
        ""#line:635
        from impala .dbapi import connect as impala #line:636
        O000O0O00OOO0OOO0 =impala (user =connect_dict ['user'],password =connect_dict ['password'],host =connect_dict ['host'],port =int (connect_dict ['port']),auth_mechanism ='PLAIN')#line:643
        O0OOO000O00O0O0OO =O000O0O00OOO0OOO0 .cursor ()#line:644
        O00000000000OOO00 =r'^insert\s|^update\s|^truncate\s|^delete\s|^load\s|^refresh\s|^upsert\s'#line:645
        O0O0OOO0O000OOO00 =None #line:646
        for O00OOOOOOO0O0O00O in OOOO0OO0OOOOO0O00 :#line:647
            print (O00OOOOOOO0O0O00O )#line:648
            O00OOOOOOO0O0O00O =O00OOOOOOO0O0O00O .strip ()#line:649
            if not O00OOOOOOO0O0O00O :#line:650
                continue #line:651
            if re .search (O00000000000OOO00 ,O00OOOOOOO0O0O00O ,re .I |re .IGNORECASE ):#line:652
                O0OOO000O00O0O0OO .execute (O00OOOOOOO0O0O00O )#line:653
            else :#line:654
                O0OOO000O00O0O0OO .execute (O00OOOOOOO0O0O00O )#line:655
                try :#line:656
                    O0O0OOO0O000OOO00 =O0OOO000O00O0O0OO .fetchall ()#line:657
                except Exception as O0OOO000OOO00000O :#line:658
                    print (O0OOO000OOO00000O .args )#line:659
        O000O0O00OOO0OOO0 .close ()#line:660
        return O0O0OOO0O000OOO00 #line:661
    def hive (OOO0O00O00OO000O0 ,O0OOOO0OOO00O000O ,connect_dict =None ):#line:663
        ""#line:664
        from impala .dbapi import connect as impala #line:665
        O0OOOOO0O0O0O0O0O =impala (user =connect_dict ['user'],password =connect_dict ['password'],host =connect_dict ['host'],port =int (connect_dict ['port']),auth_mechanism ='PLAIN')#line:672
        OOOO0OO0OO0OOO0OO =O0OOOOO0O0O0O0O0O .cursor ()#line:673
        O00000O0O0000O0O0 =r'^insert\s|^update\s|^truncate\s|^delete\s|^load\s'#line:674
        OO0OOOOO0OOOOOOOO =None #line:675
        for O0O0000OOOOOO0000 in O0OOOO0OOO00O000O :#line:676
            O0O0000OOOOOO0000 =O0O0000OOOOOO0000 .strip ()#line:677
            if not O0O0000OOOOOO0000 :#line:678
                continue #line:679
            print (O0O0000OOOOOO0000 )#line:680
            if O0O0000OOOOOO0000 .startswith ('refresh'):#line:681
                connect_dict ['port']=21050 #line:682
                OOO0O00O00OO000O0 .impala ([O0O0000OOOOOO0000 ],connect_dict =connect_dict )#line:683
            else :#line:684
                if re .search (O00000O0O0000O0O0 ,O0O0000OOOOOO0000 ,re .I |re .IGNORECASE ):#line:685
                    OOOO0OO0OO0OOO0OO .execute (O0O0000OOOOOO0000 )#line:686
                else :#line:687
                    OOOO0OO0OO0OOO0OO .execute (O0O0000OOOOOO0000 )#line:688
                    try :#line:689
                        OO0OOOOO0OOOOOOOO =OOOO0OO0OO0OOO0OO .fetchall ()#line:690
                    except Exception as OOOO00O0OO0O0OOO0 :#line:691
                        print (OOOO00O0OO0O0OOO0 .args )#line:692
        O0OOOOO0O0O0O0O0O .close ()#line:693
        return OO0OOOOO0OOOOOOOO #line:694
    def mysql (O0OOO000000O000O0 ,OOOO0O0O0OO0O000O ,connect_dict =None ):#line:696
        import pymysql #line:697
        OOO00O0O000O0O00O =pymysql .connect (user =connect_dict ['user'],password =connect_dict ['password'],host =connect_dict ['host'],port =connect_dict ['port'],database =connect_dict ['db'])#line:704
        try :#line:705
            O0O000OO00OO0OOOO =OOO00O0O000O0O00O .cursor ()#line:706
            OO0000O0O0OO000O0 =r'^insert\s|^update\s|^truncate\s|^delete\s|^load\s'#line:707
            for O000O0O0OOOO0O000 in OOOO0O0O0OO0O000O :#line:708
                O000O0O0OOOO0O000 =O000O0O0OOOO0O000 .strip ()#line:709
                if not O000O0O0OOOO0O000 :#line:710
                    continue #line:711
                print (O000O0O0OOOO0O000 )#line:712
                if re .search (OO0000O0O0OO000O0 ,O000O0O0OOOO0O000 ,re .I |re .IGNORECASE ):#line:713
                    try :#line:714
                        O0O000OO00OO0OOOO .execute (O000O0O0OOOO0O000 )#line:715
                        OOO00O0O000O0O00O .commit ()#line:716
                    except Exception as OOOOO00OO00000000 :#line:717
                        OOO00O0O000O0O00O .rollback ()#line:718
                        raise OOOOO00OO00000000 #line:719
                else :#line:720
                    O0O000OO00OO0OOOO .execute (O000O0O0OOOO0O000 )#line:721
                    O0O0O000O0000OO00 =O0O000OO00OO0OOOO .fetchall ()#line:722
                    O0O0O000O0000OO00 =[[OO00OOO0000O0OO00 [0 ]for OO00OOO0000O0OO00 in O0O000OO00OO0OOOO .description ]]+list (O0O0O000O0000OO00 )#line:723
                    return O0O0O000O0000OO00 #line:724
        except Exception as O0O000000O0OOOO00 :#line:725
            raise O0O000000O0OOOO00 #line:726
        finally :#line:727
            OOO00O0O000O0O00O .close ()#line:728
    def starrocks (O0OOO000O0O0O0O00 ,OO0O0O0OOO0O000O0 ,connect_dict =None ):#line:730
        return O0OOO000O0O0O0O00 .mysql (OO0O0O0OOO0O000O0 ,connect_dict )#line:731
    def oracle (O00O0OO00O00O0OOO ,OO00O00O00OOO0OO0 ,connect_dict =None ):#line:733
        import cx_Oracle #line:734
        O00OOOO00000O000O ='{}/{}@{}/{}'.format (connect_dict ['user'],connect_dict ['password'],connect_dict ['host'],connect_dict ['db'])#line:739
        O00000OO00O00O000 =cx_Oracle .connect (O00OOOO00000O000O )#line:740
        try :#line:741
            OO0OO0OOO00O00OOO =O00000OO00O00O000 .cursor ()#line:742
            O00O0OOO0000OOO0O =r'^insert\s|^update\s|^truncate\s|^delete\s|^comment\s'#line:743
            for OO0O00OO0000OOOO0 in OO00O00O00OOO0OO0 :#line:744
                OO0O00OO0000OOOO0 =OO0O00OO0000OOOO0 .strip ()#line:745
                if not OO0O00OO0000OOOO0 :#line:746
                    continue #line:747
                if re .search (O00O0OOO0000OOO0O ,OO0O00OO0000OOOO0 ,re .I ):#line:748
                    try :#line:749
                        OO0OO0OOO00O00OOO .execute (OO0O00OO0000OOOO0 )#line:750
                        O00000OO00O00O000 .commit ()#line:751
                    except Exception as O0O000OO0O0OOOO00 :#line:752
                        if OO0O00OO0000OOOO0 .startswith ('comment'):#line:753
                            print ('err:',OO0O00OO0000OOOO0 )#line:754
                            continue #line:755
                        O00000OO00O00O000 .rollback ()#line:756
                        raise O0O000OO0O0OOOO00 #line:757
                else :#line:758
                    OO0OO0OOO00O00OOO .execute (OO0O00OO0000OOOO0 )#line:759
                    O0OO0O000OOOO0000 =OO0OO0OOO00O00OOO .fetchall ()#line:760
                    O0OO0O000OOOO0000 =[[O00OO000O00OO0000 [0 ]for O00OO000O00OO0000 in OO0OO0OOO00O00OOO .description ]]+list (O0OO0O000OOOO0000 )#line:761
                    return O0OO0O000OOOO0000 #line:762
        except Exception as OOO0000OOO0OOO0O0 :#line:763
            raise OOO0000OOO0OOO0O0 #line:764
        finally :#line:765
            O00000OO00O00O000 .close ()#line:766
    def gp (O0OOOO00OO00O00O0 ,O000O00O000OO0OOO ,connect_dict =None ):#line:768
        import psycopg2 #line:769
        O0OO000O00000O0OO =psycopg2 .connect (user =connect_dict ['user'],password =connect_dict ['password'],host =connect_dict ['host'],port =connect_dict ['port'],database =connect_dict ['db'])#line:776
        try :#line:777
            O00O00000000O00O0 =O0OO000O00000O0OO .cursor ()#line:778
            OOOOO0O0OOOOO0O00 =r'^insert\s|^update\s|^truncate\s|^delete\s'#line:779
            for OOO0OO000OOOOO0OO in O000O00O000OO0OOO :#line:780
                OOO0OO000OOOOO0OO =OOO0OO000OOOOO0OO .strip ()#line:781
                if not OOO0OO000OOOOO0OO :#line:782
                    continue #line:783
                if re .search (OOOOO0O0OOOOO0O00 ,OOO0OO000OOOOO0OO ,re .I |re .IGNORECASE ):#line:784
                    try :#line:785
                        O00O00000000O00O0 .execute (OOO0OO000OOOOO0OO )#line:786
                        O0OO000O00000O0OO .commit ()#line:787
                    except Exception as OOO0OOOO0OO0O0000 :#line:788
                        O0OO000O00000O0OO .rollback ()#line:789
                        raise OOO0OOOO0OO0O0000 #line:790
                else :#line:791
                    O00O00000000O00O0 .execute (OOO0OO000OOOOO0OO )#line:792
                    OO0000OOO000O0OO0 =O00O00000000O00O0 .fetchall ()#line:793
                    OO0000OOO000O0OO0 =[[O00O0O00O0O0000O0 [0 ]for O00O0O00O0O0000O0 in O00O00000000O00O0 .description ]]+list (OO0000OOO000O0OO0 )#line:794
                    return OO0000OOO000O0OO0 #line:795
        except Exception as OOO0000000O00O00O :#line:796
            raise OOO0000000O00O00O #line:797
        finally :#line:798
            O0OO000O00000O0OO .close ()#line:799
    def mssql (O0OO00O0000OOO0OO ,O00OO00O000O0OO00 ,connect_dict =None ):#line:801
        import pymssql #line:802
        if connect_dict ['port']:#line:803
            O00OO0OOOO00OOO00 =pymssql .connect (user =connect_dict ['user'],password =connect_dict ['password'],host =connect_dict ['host'],port =int (connect_dict ['port']),database =connect_dict ['db'],charset ="utf8",)#line:811
        else :#line:812
            O00OO0OOOO00OOO00 =pymssql .connect (user =connect_dict ['user'],password =connect_dict ['password'],host =connect_dict ['host'],database =connect_dict ['db'],charset ="utf8",)#line:819
        try :#line:820
            OO00000O00000O0O0 =O00OO0OOOO00OOO00 .cursor ()#line:821
            OO0O0OOOOOO0O0OOO =r'^insert\s|^update\s|^truncate\s|^delete\s'#line:822
            for O0O0OOOOOOO0000OO in O00OO00O000O0OO00 :#line:823
                O0O0OOOOOOO0000OO =O0O0OOOOOOO0000OO .strip ()#line:824
                if not O0O0OOOOOOO0000OO :#line:825
                    continue #line:826
                if re .search (OO0O0OOOOOO0O0OOO ,O0O0OOOOOOO0000OO ,re .I |re .IGNORECASE ):#line:827
                    try :#line:828
                        OO00000O00000O0O0 .execute (O0O0OOOOOOO0000OO )#line:829
                        O00OO0OOOO00OOO00 .commit ()#line:830
                    except Exception as O00O0000OO0O00OO0 :#line:831
                        O00OO0OOOO00OOO00 .rollback ()#line:832
                        raise O00O0000OO0O00OO0 #line:833
                else :#line:834
                    OO00000O00000O0O0 .execute (O0O0OOOOOOO0000OO )#line:835
                    OO0OOOO00OOO00OOO =OO00000O00000O0O0 .fetchall ()#line:836
                    OO0OOOO00OOO00OOO =[[O0O0O00OOOO00O00O [0 ]for O0O0O00OOOO00O00O in OO00000O00000O0O0 .description ]]+list (OO0OOOO00OOO00OOO )#line:837
                    return OO0OOOO00OOO00OOO #line:838
        except Exception as O00OOO0000O000O00 :#line:839
            raise O00OOO0000O000O00 #line:840
        finally :#line:841
            O00OO0OOOO00OOO00 .close ()#line:842
    def execute_sql_list (O0OO000000O00OO0O ,OO000000OO00O0O0O ,db_connect ='',connect_dict =None ):#line:844
        if db_connect =='':db_connect ='oracle'#line:845
        O0OOO00OOOO00OOO0 =getattr (O0OO000000O00OO0O ,db_connect )#line:846
        return O0OOO00OOOO00OOO0 (OO000000OO00O0O0O ,connect_dict =connect_dict )#line:847
    def excute_proc (O0O0O000O000O00OO ,O00OO000OO0OOO000 ,O0O0000O00O0OO0OO ,proc_para =None ):#line:849
        import cx_Oracle #line:850
        OOO0OO0OO0OO0OOOO ='{}/{}@{}/{}'.format (O0O0000O00O0OO0OO ['user'],O0O0000O00O0OO0OO ['password'],O0O0000O00O0OO0OO ['host'],O0O0000O00O0OO0OO ['db'])#line:856
        OOO0OOOO0O0O0O000 =cx_Oracle .connect (OOO0OO0OO0OO0OOOO )#line:857
        try :#line:859
            OO0000O0OOOOO0O0O =OOO0OOOO0O0O0O000 .cursor ()#line:860
            print ("开始执行过程:{}  参数: {}".format (O00OO000OO0OOO000 ,proc_para ))#line:861
            if proc_para is None :#line:862
                OOOO00000O0OO00O0 =OO0000O0OOOOO0O0O .callproc (O00OO000OO0OOO000 )#line:863
                OOO0OOOO0O0O0O000 .commit ()#line:864
            else :#line:865
                OOOO00000O0OO00O0 =OO0000O0OOOOO0O0O .callproc (O00OO000OO0OOO000 ,proc_para )#line:867
                OOO0OOOO0O0O0O000 .commit ()#line:868
            OO0000O0OOOOO0O0O .close ()#line:869
            OOO0OOOO0O0O0O000 .close ()#line:870
            print (OOOO00000O0OO00O0 )#line:871
        except Exception as OOO00O0OOO000O000 :#line:872
            OOO0OOOO0O0O0O000 .rollback ()#line:873
            OOO0OOOO0O0O0O000 .close ()#line:874
            raise OOO00O0OOO000O000 #line:876
        return True #line:877
    def insert_contents_oracle (O0O0000OO00OO0O0O ,OOO0000OO00OOO000 ,OOOOOOO0OOOOO00O0 ,per_in =100 ,connect_dict =None ):#line:879
        import cx_Oracle #line:880
        OO000O00O0OO00OO0 ='{}/{}@{}:{}/{}'.format (connect_dict ['user'],connect_dict ['password'],connect_dict ['host'],connect_dict ['port'],connect_dict ['db'])#line:886
        O0O0OO0OO0O0OOO0O =cx_Oracle .connect (OO000O00O0OO00OO0 )#line:887
        OO0000O0000OO0O00 =O0O0OO0OO0O0OOO0O .cursor ()#line:888
        O00OO0000O00OO00O =' into {} values {}'#line:889
        O0OO00OOOOOOO0000 =''#line:890
        OOO0O000OOOOOOO00 =len (OOO0000OO00OOO000 )#line:891
        logging .info ("total {} records need to insert table {}: ".format (OOO0O000OOOOOOO00 ,OOOOOOO0OOOOO00O0 ))#line:892
        try :#line:893
            for OO0OO0000O0O0OOOO in range (OOO0O000OOOOOOO00 ):#line:894
                O0OO00OOOOOOO0000 =O0OO00OOOOOOO0000 +O00OO0000O00OO00O .format (OOOOOOO0OOOOO00O0 ,tuple (OOO0000OO00OOO000 [OO0OO0000O0O0OOOO ]))+'\n'#line:895
                if (OO0OO0000O0O0OOOO +1 )%per_in ==0 or OO0OO0000O0O0OOOO ==OOO0O000OOOOOOO00 -1 :#line:896
                    OO0O0O00O00OOOOOO ='insert all '+O0OO00OOOOOOO0000 +'select 1 from dual'#line:897
                    logging .debug (OO0O0O00O00OOOOOO )#line:898
                    OO0000O0000OO0O00 .execute (OO0O0O00O00OOOOOO )#line:899
                    O0O0OO0OO0O0OOO0O .commit ()#line:900
                    O0OO00OOOOOOO0000 =''#line:901
            return str (OOO0O000OOOOOOO00 )#line:902
        except Exception as O0O0O0O000OOO0000 :#line:903
            try :#line:904
                O0O0OO0OO0O0OOO0O .rollback ()#line:905
                OO0000O0000OO0O00 .execute ("delete from {} where UPLOADTIME = '{}'".format (OOOOOOO0OOOOO00O0 ,OOO0000OO00OOO000 [0 ][-1 ]))#line:906
                O0O0OO0OO0O0OOO0O .commit ()#line:907
            except Exception :#line:908
                logging .error ('can not delete by uploadtime')#line:909
            finally :#line:910
                raise O0O0O0O000OOO0000 #line:911
        finally :#line:912
            O0O0OO0OO0O0OOO0O .close ()#line:913
class _OO00O0OOO000O00O0 (object ):#line:917
    connect =None #line:918
    def __init__ (OO0OOO00O0O0OOOOO ,O00OOOO00000OO000 ,O00OO0OO00OO0O000 ,OOO00O00O0OOO0000 ,offsetDict =None ):#line:920
        OO0OOO00O0O0OOOOO .end_offset =None #line:921
        OO0OOO00O0O0OOOOO .db_err_count =0 #line:922
        OO0OOO00O0O0OOOOO .topic =O00OOOO00000OO000 #line:923
        OO0OOO00O0O0OOOOO .kafkaconfig =O00OO0OO00OO0O000 #line:924
        OO0OOO00O0O0OOOOO .offsetDict ={}#line:925
        OO0OOO00O0O0OOOOO .current_dir =OOO00O00O0OOO0000 #line:926
        OO0OOO00O0O0OOOOO .offsetDict =offsetDict #line:927
        try :#line:928
            OO0OOO00O0O0OOOOO .consumer =OO0OOO00O0O0OOOOO .connect_kafka_customer ()#line:929
        except Exception as OO00OO0O0OOO0O00O :#line:930
            OO00OO0O0OOO0O00O ='kafka无法连接','ErrLocation：{}\n'.format (O00OOOO00000OO000 )+str (OO00OO0O0OOO0O00O )+',kafka消费者无法创建'#line:931
            raise OO00OO0O0OOO0O00O #line:932
    def get_toggle_or_offset (OO0O0O0OOOO00OO0O ,OO0O000OOOOO00O0O ,O0000OOOOOO0O0O0O ):#line:934
        ""#line:935
        if OO0O0O0OOOO00OO0O .offsetDict :#line:936
            if isinstance (OO0O0O0OOOO00OO0O .offsetDict ,int ):#line:937
                return OO0O0O0OOOO00OO0O .offsetDict #line:938
            else :#line:939
                O0O0O00OOO00OOOOO =OO0O0O0OOOO00OO0O .offsetDict .get (str (O0000OOOOOO0O0O0O ))#line:940
                if O0O0O00OOO00OOOOO is not None :#line:941
                    return O0O0O00OOO00OOOOO #line:942
        O0O0O00OOO00OOOOO =0 #line:943
        try :#line:944
            OOO0O0000O0O000OO =f"{OO0O0O0OOOO00OO0O.current_dir}/kafka/{OO0O000OOOOO00O0O}_offset_{O0000OOOOOO0O0O0O}.txt"#line:945
            if os .path .exists (OOO0O0000O0O000OO ):#line:946
                O000O0O0O000OOOO0 =open (OOO0O0000O0O000OO ).read ()#line:947
                if O000O0O0O000OOOO0 :#line:948
                    O0O0O00OOO00OOOOO =int (O000O0O0O000OOOO0 )#line:949
            else :#line:950
                with open (OOO0O0000O0O000OO ,encoding ='utf-8',mode ='a')as O0OO0O0OO000O0O0O :#line:951
                    O0OO0O0OO000O0O0O .close ()#line:952
        except Exception as OO000OOOO0OOO0OOO :#line:953
            print (f"读取失败：{OO000OOOO0OOO0OOO}")#line:954
            raise OO000OOOO0OOO0OOO #line:955
        return O0O0O00OOO00OOOOO #line:956
    def write_offset (OOOO0OOO0OOO00000 ,OO0O00OO0O00O00OO ,offset =None ):#line:958
        ""#line:961
        if OOOO0OOO0OOO00000 .topic and offset :#line:962
            OOO0OOO000OO0OO0O =f"{OOOO0OOO0OOO00000.current_dir}/kafka/{OOOO0OOO0OOO00000.topic}_offset_{OO0O00OO0O00O00OO}.txt"#line:964
            try :#line:965
                with open (OOO0OOO000OO0OO0O ,'w')as OO000O000O0O0OOOO :#line:966
                    OO000O000O0O0OOOO .write (str (offset ))#line:967
                    OO000O000O0O0OOOO .close ()#line:968
            except Exception as OO0O000O0OO0O0O0O :#line:969
                print (f"写入offset出错：{OO0O000O0OO0O0O0O}")#line:970
                raise OO0O000O0OO0O0O0O #line:971
    def connect_kafka_customer (O0OOOO00O0O000O00 ):#line:973
        ""#line:974
        OO0OO00O000O0O000 =KafkaConsumer (**O0OOOO00O0O000O00 .kafkaconfig )#line:975
        return OO0OO00O000O0O000 #line:976
    def parse_data (O0OOO000OO0000000 ,OO0000O000O000O0O ):#line:978
        ""#line:983
        return dict ()#line:984
    def gen_sql (OOO00OOO0O0O00OO0 ,O000OO00O000O0OO0 ):#line:986
        ""#line:991
        O0000000OOO0O0000 =[]#line:992
        for O00OOO0O0OO0OO0OO in O000OO00O000O0OO0 :#line:993
            O0000000OOO0O0000 .append (str (tuple (O00OOO0O0OO0OO0OO )))#line:995
        return ','.join (O0000000OOO0O0000 )#line:996
    def dispose_kafka_data (OO000OO0OO0O000OO ,OO00OOO000O0OO0OO ):#line:998
        ""#line:1003
        pass #line:1004
    def get_now_time (OO000OOOO0O0OOO00 ):#line:1006
        ""#line:1010
        O000000O0O0OO0OOO =int (time .time ())#line:1011
        return time .strftime ('%Y-%m-%d %H:%M:%S',time .localtime (O000000O0O0OO0OOO ))#line:1012
    def tran_data (O000OOOO000O0OOOO ,O00OOOOOOO00OOOOO ,OOO00O0OOO0OOOO0O ):#line:1014
        ""#line:1020
        O0OOOO0000OOO0000 =O00OOOOOOO00OOOOO .get (OOO00O0OOO0OOOO0O ,"")#line:1021
        O0OOOO0000OOO0000 =""if O0OOOO0000OOO0000 is None else O0OOOO0000OOO0000 #line:1022
        return str (O0OOOO0000OOO0000 )#line:1023
    def consumer_data (O00OOOOOOO00000O0 ,OOOO0OOOO0OO00O00 ,O0O00000O0O00O00O ,OO0O000O0O0O0OOO0 ):#line:1025
        ""#line:1032
        if O00OOOOOOO00000O0 .consumer :#line:1033
            O00OOOOOOO00000O0 .consumer .assign ([TopicPartition (topic =O00OOOOOOO00000O0 .topic ,partition =OOOO0OOOO0OO00O00 )])#line:1034
            O000O0O00O0O00OOO =TopicPartition (topic =O00OOOOOOO00000O0 .topic ,partition =OOOO0OOOO0OO00O00 )#line:1036
            OOO000O0OO00O0OOO =O00OOOOOOO00000O0 .consumer .beginning_offsets ([O000O0O00O0O00OOO ])#line:1037
            OO0O0OOO0O0OO0O00 =OOO000O0OO00O0OOO .get (O000O0O00O0O00OOO )#line:1038
            O000OOO0O00OOO0O0 =O00OOOOOOO00000O0 .consumer .end_offsets ([O000O0O00O0O00OOO ])#line:1039
            OO0O0OOO00O0000OO =O000OOO0O00OOO0O0 .get (O000O0O00O0O00OOO )#line:1040
            print (f'建立消费者, {OOOO0OOOO0OO00O00}分区, 最小offset:{OO0O0OOO0O0OO0O00}, 最大offset:{OO0O0OOO00O0000OO}')#line:1041
            if O0O00000O0O00O00O <OO0O0OOO0O0OO0O00 :#line:1042
                print (f'Warning: 消费offset：{O0O00000O0O00O00O} 小于最小offset:{OO0O0OOO0O0OO0O00}')#line:1043
                O0O00000O0O00O00O =OO0O0OOO0O0OO0O00 #line:1044
            if O0O00000O0O00O00O >=OO0O0OOO00O0000OO :#line:1045
                print (f'消费offset：{O0O00000O0O00O00O} 大于最大offset:{OO0O0OOO00O0000OO}, 本次不消费')#line:1046
                if O0O00000O0O00O00O ==996 :#line:1047
                    O0O00000O0O00O00O =OO0O0OOO00O0000OO -1 #line:1048
                else :#line:1049
                    return #line:1050
            O00OOOOOOO00000O0 .end_offset =OO0O0OOO00O0000OO #line:1051
            O00OOOOOOO00000O0 .consumer .seek (TopicPartition (topic =O00OOOOOOO00000O0 .topic ,partition =OOOO0OOOO0OO00O00 ),offset =O0O00000O0O00O00O )#line:1052
            print (f'消费{OOOO0OOOO0OO00O00}分区, 开始消费offset：{O0O00000O0O00O00O}!')#line:1053
            O0OOOOO0OO0OO00O0 =OO0O000O0O0O0OOO0 (O00OOOOOOO00000O0 .consumer )#line:1054
            O0O00000O0O00O00O =O0OOOOO0OO0OO00O0 .offset +1 #line:1055
            O00OOOOOOO00000O0 .offsetDict [OOOO0OOOO0OO00O00 ]=O0O00000O0O00O00O #line:1058
            O00OOOOOOO00000O0 .write_offset (OOOO0OOOO0OO00O00 ,O0O00000O0O00O00O )#line:1059
            O00OOOOOOO00000O0 .offsetDict [OOOO0OOOO0OO00O00 ]=O0O00000O0O00O00O #line:1062
            O00OOOOOOO00000O0 .write_offset (OOOO0OOOO0OO00O00 ,O0O00000O0O00O00O )#line:1063
    def consumer_topic (OOO00O0OOOO00OOO0 ,OO00O00OOO0000O0O ):#line:1065
        print (f"topic: {OOO00O0OOOO00OOO0.topic}")#line:1066
        print ('开始解析。')#line:1067
        OO0OOOO000O0O00O0 =OOO00O0OOOO00OOO0 .consumer .partitions_for_topic (topic =OOO00O0OOOO00OOO0 .topic )#line:1069
        for OO0OOO00OOOOOO00O in OO0OOOO000O0O00O0 :#line:1070
            O0OOOOOOOO0OOOO0O =OOO00O0OOOO00OOO0 .get_toggle_or_offset (OOO00O0OOOO00OOO0 .topic ,OO0OOO00OOOOOO00O )#line:1071
            O000O0OOO00OO000O =0 if O0OOOOOOOO0OOOO0O <0 else O0OOOOOOOO0OOOO0O #line:1073
            OOO00O0OOOO00OOO0 .consumer_data (OO0OOO00OOOOOO00O ,O000O0OOO00OO000O ,OO00O00OOO0000O0O )#line:1074
    def save_all_offset (O0OOO0OO0O0OO0OOO ):#line:1076
        for OO00O000OO0OOOOO0 ,O00OO0O000O00OO0O in O0OOO0OO0O0OO0OOO .offsetDict :#line:1077
            O0OOO0OO0O0OO0OOO .write_offset (OO00O000OO0OOOOO0 ,O00OO0O000O00OO0O )#line:1078



from airflow.contrib.dagsetup.gen_airflow_dagfile import gen_airflow_dag,rm_airflow_dag,gen_airflow_config

def gen_dag_file(*args,**kwargs):
    '''generate the dags and script'''
    gen_airflow_dag(*args,**kwargs)

def rm_dag_file(name):
    rm_airflow_dag(name)

def gen_dag_config(name):
    return gen_airflow_config(name)



import requests
import json


def get_header():
    import base64
    name = 'admin'
    pwd = 'admin'
    auth = str(base64.b64encode(f'{name}:{pwd}'.encode('utf-8')), 'utf-8')
    header = {
        'Content-Type': 'application/json',
        'Authorization': f'Basic {auth}'
    }
    return header


def t_create_dagfile():
    owner = 'john'
    schedule_interval = '10 08 * * *'
    # schedule_interval = '@once'
    schedule_rate = 'day'  # 每周,每月
    project_name = 'P2'  # which is used as id and file name
    # the job will execute in dag by sequence as listed, first item is to use what driver to execute
    # 数据库及文件类型,文件命名,参数
    job_list = '''
    #hive_sql predict para1,para2
    #impala_sql center
    #hive_sql huli
    #impala_sql test1
    #ktr test para3
    #/diy diy1
    #trigger trigger2  targetdag2
    #/branch  branch1
    #hdfs abcc
    '''
    dag_sequence = '''
center >> test1 >> trigger2
test
    '''
    retries = 5
    dag_remark = 'finance'
    kwargs = {
        'owner': owner,
        'project_name': project_name,
        'schedule_interval': schedule_interval,
        'schedule_rate': schedule_rate,
        'job_list': job_list,
        'tags': 'test1,test2',
        'dag_sequence': dag_sequence,
        'dag_remark': '##dag_remark',
        'description': 'description'
    }
    # gen_airflow_dag(**kwargs)
    # url = 'http://0.0.0.0:8080/api/experimental/create_dagfile?token=jzbuxdr!wkh1_dhknzxhpo1t7no_bw*ar!t$pb5wqncrr4_k3y'
    # response = requests.post(url,data=json.dumps(kwargs),headers={'Content-Type': 'application/json'})
    url = 'http://0.0.0.0:8080/api/v1/create_dagfile'
    # url = 'http://10.10.1.184:8080/api/v1/create_dagfile'
    response = requests.post(url, data=json.dumps(kwargs), headers=get_header())
    print(response.text)


def t_save_etlfile():
    url = 'http://0.0.0.0:8080/api/v1/create_etlfile'
    # url = 'http://10.10.1.184:8080/api/v1/create_dagfile'
    kwargs = {
        'project_name': 't_etlfile',
        'filename': 'p1',
        'driver': 'impala_sql',
        'txt': 'select * from abc'
    }
    response = requests.post(url, data=json.dumps(kwargs), headers=get_header())
    print(response.text)

def t_get_etlfile():
    url = 'http://0.0.0.0:8080/api/v1/gen_etlconfig'
    kwargs = {
        'name': 't_etlfile',
        'filename': 'p1',
        'driver': 'impala_sql'
    }
    response = requests.get(url=url, params=kwargs, headers=get_header())
    print(response.text)


if __name__ == '__main__':
    # t_save_etlfile()
    t_get_etlfile()
